@@ -15,11 +15,23 @@ end
 
 open Let_syntax
 
-module Only_packages = struct
+module Only_packages : sig
+  type t = private
+    { names : Dune_engine.Package.Name.Set.t
+    ; command_line_option : string
+    }
+
+  val create :
+    names:Dune_engine.Package.Name.Set.t -> command_line_option:string -> t
+end = struct
   type t =
     { names : Dune_engine.Package.Name.Set.t
     ; command_line_option : string
     }
+
+  let create ~names ~command_line_option =
+    Clflags.only_packages := Some names;
+    { names; command_line_option }
 end
 
 type t =
@@ -74,9 +86,31 @@ let prefix_target common s = common.target_prefix ^ s
 
 let instrument_with t = t.instrument_with
 
+(* To avoid needless recompilations under Windows, normalize the drive letter to
+   uppercase. *)
+let normalize_path path =
+  if Sys.win32 then
+    let src = Path.External.to_string path in
+    let is_letter = function
+      | 'a' .. 'z'
+      | 'A' .. 'Z' ->
+        true
+      | _ -> false
+    in
+    if String.length src >= 2 && is_letter src.[0] && src.[1] = ':' then (
+      let dst = Bytes.create (String.length src) in
+      Bytes.set dst 0 (Char.uppercase_ascii src.[0]);
+      Bytes.blit_string ~src ~src_pos:1 ~dst ~dst_pos:1
+        ~len:(String.length src - 1);
+      Path.External.of_string (Bytes.unsafe_to_string dst)
+    ) else
+      path
+  else
+    path
+
 let set_dirs c =
   if c.root.dir <> Filename.current_dir_name then Sys.chdir c.root.dir;
-  Path.set_root (Path.External.cwd ());
+  Path.set_root (normalize_path (Path.External.cwd ()));
   Path.Build.set_build_dir (Path.Build.Kind.of_string c.build_dir)
 
 let set_common_other ?log_file c ~targets =
@@ -118,6 +152,12 @@ let footer =
 
 let copts_sect = "COMMON OPTIONS"
 
+let debug_backtraces =
+  Arg.(
+    value & flag
+    & info [ "debug-backtraces" ] ~docs:copts_sect
+        ~doc:{|Always print exception backtraces.|})
+
 let examples = function
   | [] -> `Blocks []
   | _ :: _ as examples ->
@@ -184,12 +224,12 @@ let build_info =
              , ver_string (B.Statically_linked_library.version lib) ))
       |> List.sort ~compare:Poly.compare
     in
-    ( match libs with
+    (match libs with
     | [] -> ()
     | _ ->
       pr "statically linked libraries:";
       let longest = String.longest_map libs ~f:fst in
-      List.iter libs ~f:(fun (name, v) -> pr "- %-*s %s" longest name v) );
+      List.iter libs ~f:(fun (name, v) -> pr "- %-*s %s" longest name v));
     exit 0
   )
 
@@ -287,7 +327,10 @@ module Options_implied_by_dash_p = struct
       let doc =
         "Promote the generated <package>.install files to the source tree"
       in
-      Arg.(value & flag & info [ "promote-install-files" ] ~docs ~doc)
+      Arg.(
+        last
+        & opt_all ~vopt:true bool [ false ]
+        & info [ "promote-install-files" ] ~docs ~doc)
     in
     { root
     ; only_packages = None
@@ -299,36 +342,34 @@ module Options_implied_by_dash_p = struct
     ; promote_install_files
     }
 
-  let release_options =
-    { root = Some "."
-    ; only_packages = None
-    ; ignore_promoted_rules = true
-    ; config_file = No_config
-    ; profile = Some Profile.Release
-    ; default_target =
-        Arg.Dep.alias_rec ~dir:Path.Local.root Dune_engine.Alias.Name.install
-    ; always_show_command_line = true
-    ; promote_install_files = true
-    }
-
   let dash_dash_release =
-    let+ (_ : bool) =
-      Arg.(
-        value & flag
-        & info [ "release" ] ~docs ~docv:"PACKAGES"
-            ~doc:
-              "Put $(b,dune) into a reproducible $(i,release) mode. This is in \
-               fact a shorthand for $(b,--root . --ignore-promoted-rules \
-               --no-config --profile release --always-show-command-line \
-               --promote-install-files --default-target @install). You should \
-               use this option for release builds. For instance, you must use \
-               this option in your $(i,<package>.opam) files. Except if you \
-               already use $(b,-p), as $(b,-p) implies this option.")
-    in
-    release_options
+    Arg.(
+      value
+      & alias
+          [ "--root"
+          ; "."
+          ; "--ignore-promoted-rules"
+          ; "--no-config"
+          ; "--profile"
+          ; "release"
+          ; "--always-show-command-line"
+          ; "--promote-install-files"
+          ; "--default-target"
+          ; "@install"
+          ]
+      & info [ "release" ] ~docs ~docv:"PACKAGES"
+          ~doc:
+            "Put $(b,dune) into a reproducible $(i,release) mode. This is in \
+             fact a shorthand for $(b,--root . --ignore-promoted-rules \
+             --no-config --profile release --always-show-command-line \
+             --promote-install-files --default-target @install). You should \
+             use this option for release builds. For instance, you must use \
+             this option in your $(i,<package>.opam) files. Except if you \
+             already use $(b,-p), as $(b,-p) implies this option.")
 
   let options =
-    let+ t = one_of options dash_dash_release
+    let+ t = options
+    and+ _ = dash_dash_release
     and+ only_packages =
       let+ names =
         Arg.(
@@ -345,41 +386,34 @@ module Options_implied_by_dash_p = struct
                       build a particular $(b,<package>.install) target.|})
       in
       Option.map names ~f:(fun names ->
-          { Only_packages.names; command_line_option = "only-packages" })
+          Only_packages.create ~names ~command_line_option:"only-packages")
     in
     { t with only_packages }
 
   let dash_p =
-    let+ pkgs, args =
-      Term.with_used_args
-        Arg.(
-          value
-          & opt (some packages) None
-          & info
-              [ "p"; "for-release-of-packages" ]
-              ~docs ~docv:"PACKAGES"
-              ~doc:
-                "Shorthand for $(b,--release --only-packages PACKAGE). You \
-                 must use this option in your $(i,<package>.opam) files, in \
-                 order to build only what's necessary when your project \
-                 contains multiple packages as well as getting reproducible \
-                 builds.")
-    in
-    { release_options with
-      only_packages =
-        Option.map pkgs ~f:(fun names ->
-            { Only_packages.names; command_line_option = List.hd args })
-    }
+    Term.with_used_args
+      Arg.(
+        value
+        & alias_opt (fun s -> [ "--release"; "--only-packages"; s ])
+        & info
+            [ "p"; "for-release-of-packages" ]
+            ~docs ~docv:"PACKAGES"
+            ~doc:
+              "Shorthand for $(b,--release --only-packages PACKAGE). You must \
+               use this option in your $(i,<package>.opam) files, in order to \
+               build only what's necessary when your project contains multiple \
+               packages as well as getting reproducible builds.")
 
   let term =
-    let+ t = one_of options dash_p
+    let+ t = options
+    and+ _ = dash_p
     and+ profile =
       let doc =
         "Build profile. $(b,dev) if unspecified or $(b,release) if -p is set."
       in
       Arg.(
-        value
-        & opt (some profile) None
+        last
+        & opt_all (some profile) [ None ]
         & info [ "profile" ] ~docs
             ~env:(Arg.env_var ~doc "DUNE_PROFILE")
             ~doc:
@@ -475,11 +509,7 @@ let term =
     Arg.(
       value & flag
       & info [ "debug-findlib" ] ~docs ~doc:{|Debug the findlib sub-system.|})
-  and+ debug_backtraces =
-    Arg.(
-      value & flag
-      & info [ "debug-backtraces" ] ~docs
-          ~doc:{|Always print exception backtraces.|})
+  and+ debug_backtraces = debug_backtraces
   and+ debug_artifact_substitution =
     Arg.(
       value & flag
@@ -487,14 +517,19 @@ let term =
           [ "debug-artifact-substitution" ]
           ~docs ~doc:"Print debugging info about artifact substitution")
   and+ terminal_persistence =
+    let modes = Config.Terminal_persistence.all in
+    let doc =
+      let f s = fst s |> Printf.sprintf "$(b,%s)" in
+      Printf.sprintf
+        {|Changes how the log of build results are displayed to the
+          console between rebuilds while in $(b,--watch) mode. Supported modes:
+          %s.|}
+        (List.map ~f modes |> String.concat ~sep:", ")
+    in
     Arg.(
       value
-      & opt (some (enum Config.Terminal_persistence.all)) None
-      & info [ "terminal-persistence" ] ~docs ~docv:"MODE"
-          ~doc:
-            {|
-         Changes how the log of build results are displayed to the
-         console between rebuilds while in --watch mode. |})
+      & opt (some (enum modes)) None
+      & info [ "terminal-persistence" ] ~docs ~docv:"MODE" ~doc)
   and+ display = display_term
   and+ no_buffer =
     let doc =
