@@ -13,7 +13,7 @@ val root : t -> Workspace_root.t
 val config : t -> Dune_engine.Config.t
 
 module Only_packages : sig
-  type t =
+  type t = private
     { names : Dune_engine.Package.Name.Set.t
     ; command_line_option : string
           (** Which of [-p], [--only-packages], ... was passed *)
@@ -60,6 +60,8 @@ val footer : Cmdliner.Manpage.block
 
 val term : t Cmdliner.Term.t
 
+val debug_backtraces : bool Cmdliner.Term.t
+
 val config_term : Dune_engine.Config.t Cmdliner.Term.t
 
 val display_term : Dune_engine.Config.Display.t option Cmdliner.Term.t
