@@ -45,7 +45,8 @@ let term =
   and+ args = Arg.(value & pos_right 0 string [] (Arg.info [] ~docv:"ARGS")) in
   Common.set_common common ~targets:[ Arg.Dep.file prog ];
   let setup = Scheduler.go ~common (fun () -> Import.Main.setup common) in
-  let context = Import.Main.find_context_exn setup.workspace ~name:context in
+  let sctx = Import.Main.find_scontext_exn setup ~name:context in
+  let context = Dune_rules.Super_context.context sctx in
   let path_relative_to_build_root p =
     Common.prefix_target common p
     |> Path.Build.relative context.build_dir
@@ -59,31 +60,31 @@ let term =
   in
   let targets =
     lazy
-      ( ( match prog_where with
-        | `Search p ->
-          [ Path.Build.relative
-              (Config.local_install_bin_dir ~context:context.name)
-              p
-            |> Path.build
-          ]
-        | `This_rel p when Sys.win32 ->
-          [ p; Path.extend_basename p ~suffix:Bin.exe ]
-        | `This_rel p -> [ p ]
-        | `This_abs p when Path.is_in_build_dir p -> [ p ]
-        | `This_abs _ -> [] )
+      ((match prog_where with
+       | `Search p ->
+         [ Path.Build.relative
+             (Config.local_install_bin_dir ~context:context.name)
+             p
+           |> Path.build
+         ]
+       | `This_rel p when Sys.win32 ->
+         [ p; Path.extend_basename p ~suffix:Bin.exe ]
+       | `This_rel p -> [ p ]
+       | `This_abs p when Path.is_in_build_dir p -> [ p ]
+       | `This_abs _ -> [])
       |> List.map ~f:(fun p -> Target.Path p)
       |> Target.resolve_targets_mixed common setup
       |> List.concat_map ~f:(function
            | Ok targets -> targets
-           | Error _ -> []) )
+           | Error _ -> []))
   in
   let real_prog =
-    ( if not no_rebuild then
+    (if not no_rebuild then
       match Lazy.force targets with
       | [] -> ()
       | targets ->
         Scheduler.go ~common (fun () -> do_build targets);
-        Hooks.End_of_build.run () );
+        Hooks.End_of_build.run ());
     match prog_where with
     | `Search prog ->
       let path =
@@ -124,13 +125,14 @@ let term =
             "Program %S isn't built yet. You need to build it first or remove \
              the --no-build option."
             prog
-        ] )
+        ])
   | None, false ->
     let hints = hints () in
     User_error.raise ~hints [ Pp.textf "Program %S not found!" prog ]
   | Some real_prog, _ ->
     let real_prog = Path.to_string real_prog in
     let argv = prog :: args in
-    restore_cwd_and_execve common real_prog argv context.env
+    restore_cwd_and_execve common real_prog argv
+      (Super_context.context_env sctx)
 
 let command = (term, info)
