@@ -34,12 +34,12 @@ let resolve_package_install setup pkg =
       ~hints:
         (User_message.did_you_mean pkg
            ~candidates:
-             ( Package.Name.Map.keys setup.conf.packages
-             |> List.map ~f:Package.Name.to_string ))
+             (Package.Name.Map.keys setup.conf.packages
+             |> List.map ~f:Package.Name.to_string))
 
 let print_unix_error f =
-  try f ()
-  with Unix.Unix_error (e, _, _) ->
+  try f () with
+  | Unix.Unix_error (e, _, _) ->
     User_message.prerr (User_error.make [ Pp.text (Unix.error_message e) ])
 
 module Special_file = struct
@@ -68,6 +68,7 @@ module type File_operations = sig
     -> executable:bool
     -> special_file:Special_file.t option
     -> package:Package.Name.t
+    -> conf:Dune_rules.Artifact_substitution.conf
     -> unit Fiber.t
 
   val mkdir_p : Path.t -> unit
@@ -82,7 +83,7 @@ module type Workspace = sig
 end
 
 module File_ops_dry_run : File_operations = struct
-  let copy_file ~src ~dst ~executable ~special_file:_ ~package:_ =
+  let copy_file ~src ~dst ~executable ~special_file:_ ~package:_ ~conf:_ =
     Format.printf "Copying %s to %s (executable: %b)\n"
       (Path.to_string_maybe_quoted src)
       (Path.to_string_maybe_quoted dst)
@@ -119,14 +120,17 @@ module File_ops_real (W : Workspace) : File_operations = struct
     match f ic with
     | exception _ ->
       User_warning.emit ~loc:(Loc.in_file src)
-        [ Pp.text "Failed to parse file, not adding version information." ];
+        [ Pp.text
+            "Failed to parse file, not adding version and locations \
+             information."
+        ];
       plain_copy ()
     | No_version_needed -> plain_copy ()
     | Need_version print -> (
       match
         let open Option.O in
         let* package = Package.Name.Map.find workspace.conf.packages package in
-        get_vcs package.path
+        Package.dir package |> get_vcs
       with
       | None -> plain_copy ()
       | Some vcs ->
@@ -134,7 +138,7 @@ module File_ops_real (W : Workspace) : File_operations = struct
         let+ version = Dune_engine.Vcs.describe vcs in
         let ppf = Format.formatter_of_out_channel oc in
         print ppf ~version;
-        Format.pp_print_flush ppf () )
+        Format.pp_print_flush ppf ())
 
   let process_meta ic =
     let lb = Lexing.from_channel ic in
@@ -148,7 +152,8 @@ module File_ops_real (W : Workspace) : File_operations = struct
               raise_notrace Exit)
         in
         false
-      with Exit -> true
+      with
+      | Exit -> true
     in
     if not need_more_versions then
       No_version_needed
@@ -159,14 +164,44 @@ module File_ops_real (W : Workspace) : File_operations = struct
             Dune_rules.Meta.add_versions meta ~get_version:(fun _ ->
                 Some version)
           in
-          Pp.render_ignore_tags ppf (Dune_rules.Meta.pp meta.entries))
+          Pp.to_fmt ppf (Dune_rules.Meta.pp meta.entries))
 
-  let process_dune_package ic =
+  let replace_sites
+      ~(get_location : Dune_engine.Section.t -> Package.Name.t -> Stdune.Path.t)
+      dp =
+    match
+      List.find_map dp ~f:(function
+        | Dune_lang.List [ Atom (A "name"); Atom (A name) ] -> Some name
+        | _ -> None)
+    with
+    | None -> dp
+    | Some name ->
+      List.map dp ~f:(function
+        | Dune_lang.List ((Atom (A "sites") as sexp_sites) :: sites) ->
+          let sites =
+            List.map sites ~f:(function
+              | Dune_lang.List [ (Atom (A section) as section_sexp); _ ] ->
+                let path =
+                  get_location
+                    (Option.value_exn (Section.of_string section))
+                    (Package.Name.of_string name)
+                in
+                let open Dune_lang.Encoder in
+                pair sexp string (section_sexp, Path.to_absolute_filename path)
+              | _ -> assert false)
+          in
+          Dune_lang.List (sexp_sites :: sites)
+        | x -> x)
+
+  let process_dune_package ~get_location ic =
     let lb = Lexing.from_channel ic in
     let dp =
       Dune_lang.Parser.parse ~mode:Many lb
       |> List.map ~f:Dune_lang.Ast.remove_locs
     in
+    (* replace sites with external path in the file *)
+    let dp = replace_sites ~get_location dp in
+    (* replace version if needed in the file *)
     if
       List.exists dp ~f:(function
         | Dune_lang.List (Atom (A "version") :: _)
@@ -197,7 +232,8 @@ module File_ops_real (W : Workspace) : File_operations = struct
               Format.pp_print_cut ppf ());
           Format.pp_close_box ppf ())
 
-  let copy_file ~src ~dst ~executable ~special_file ~package =
+  let copy_file ~src ~dst ~executable ~special_file ~package
+      ~(conf : Dune_rules.Artifact_substitution.conf) =
     let chmod =
       if executable then
         fun _ ->
@@ -215,9 +251,10 @@ module File_ops_real (W : Workspace) : File_operations = struct
         match (special_file : Special_file.t option) with
         | Some META -> copy_special_file ~src ~package ~ic ~oc ~f:process_meta
         | Some Dune_package ->
-          copy_special_file ~src ~package ~ic ~oc ~f:process_dune_package
+          copy_special_file ~src ~package ~ic ~oc
+            ~f:(process_dune_package ~get_location:conf.get_location)
         | None ->
-          Dune_engine.Artifact_substitution.copy ~get_vcs ~input_file:src
+          Dune_rules.Artifact_substitution.copy ~conf ~input_file:src
             ~input:(input ic) ~output:(output oc))
 
   let remove_if_exists dst =
@@ -243,13 +280,12 @@ end
 module Sections = struct
   type t =
     | All
-    | Only of Install.Section.Set.t
+    | Only of Section.Set.t
 
-  let sections_conv : Install.Section.t list Cmdliner.Arg.converter =
+  let sections_conv : Section.t list Cmdliner.Arg.converter =
     let all =
-      Install.Section.all |> Install.Section.Set.to_list
-      |> List.map ~f:(fun section ->
-             (Install.Section.to_string section, section))
+      Section.all |> Section.Set.to_list
+      |> List.map ~f:(fun section -> (Section.to_string section, section))
     in
     Arg.list ~sep:',' (Arg.enum all)
 
@@ -261,32 +297,40 @@ module Sections = struct
     in
     match sections with
     | None -> All
-    | Some sections -> Only (Install.Section.Set.of_list sections)
+    | Some sections -> Only (Section.Set.of_list sections)
 
   let should_install t section =
     match t with
     | All -> true
-    | Only set -> Install.Section.Set.mem set section
+    | Only set -> Section.Set.mem set section
 end
 
 let file_operations ~dry_run ~workspace : (module File_operations) =
   if dry_run then
     (module File_ops_dry_run)
   else
-    ( module File_ops_real (struct
+    (module File_ops_real (struct
       let workspace = workspace
-    end) )
+    end))
 
 let package_is_vendored (pkg : Dune_engine.Package.t) =
-  match Dune_engine.File_tree.find_dir pkg.path with
-  | None -> assert false
-  | Some d -> (
-    match Dune_engine.File_tree.Dir.status d with
-    | Vendored -> true
-    | _ -> false )
+  let dir = Package.dir pkg in
+  Dune_engine.File_tree.is_vendored dir
+
+type what =
+  | Install
+  | Uninstall
+
+let pp_what fmt = function
+  | Install -> Format.pp_print_string fmt "Install"
+  | Uninstall -> Format.pp_print_string fmt "Uninstall"
+
+let cmd_what = function
+  | Install -> "install"
+  | Uninstall -> "uninstall"
 
 let install_uninstall ~what =
-  let doc = sprintf "%s packages." (String.capitalize what) in
+  let doc = Format.asprintf "%a packages." pp_what what in
   let name_ = Arg.info [] ~docv:"PACKAGE" in
   let term =
     let+ common = Common.term
@@ -323,11 +367,37 @@ let install_uninstall ~what =
         "When passed, manually override the directory to install man pages"
       in
       Arg.(value & opt (some string) None & info [ "mandir" ] ~docv:"PATH" ~doc)
+    and+ docdir =
+      let doc =
+        "When passed, manually override the directory to install documentation"
+      in
+      Arg.(value & opt (some string) None & info [ "docdir" ] ~docv:"PATH" ~doc)
+    and+ etcdir =
+      let doc =
+        "When passed, manually override the directory to install configuration \
+         files"
+      in
+      Arg.(value & opt (some string) None & info [ "etcdir" ] ~docv:"PATH" ~doc)
     and+ dry_run =
       Arg.(
         value & flag
         & info [ "dry-run" ]
             ~doc:"Only display the file operations that would be performed.")
+    and+ relocatable =
+      Arg.(
+        value & flag
+        & info [ "relocatable" ]
+            ~doc:
+              "Make the binaries relocatable (the installation directory can \
+               be moved).")
+    and+ create_install_files =
+      Arg.(
+        value & flag
+        & info [ "create-install-files" ]
+            ~doc:
+              "Do not directly install, but create install files in the root \
+               directory and create substituted files if needed in destdir \
+               (_destdir by default).")
     and+ pkgs = Arg.(value & pos_all package_name [] name_)
     and+ context =
       Arg.(
@@ -338,7 +408,7 @@ let install_uninstall ~what =
               "Select context to install from. By default, install files from \
                all defined contexts.")
     and+ sections = Sections.term in
-    Common.set_common common ~targets:[];
+    Common.set_common ~log_file:No_log_file common ~targets:[];
     Scheduler.go ~common (fun () ->
         let open Fiber.O in
         let* workspace = Import.Main.scan_workspace common in
@@ -355,7 +425,8 @@ let install_uninstall ~what =
                 if package_is_vendored pkg then
                   acc
                 else
-                  pkg.name :: acc)
+                  let name = Package.name pkg in
+                  name :: acc)
             |> List.rev
           | l -> l
         in
@@ -379,9 +450,9 @@ let install_uninstall ~what =
                   Pp.text (Path.to_string p))
             ]
             ~hints:[ Pp.text "try running: dune build @install" ];
-        ( match
-            (contexts, prefix_from_command_line, libdir_from_command_line)
-          with
+        (match
+           (contexts, prefix_from_command_line, libdir_from_command_line)
+         with
         | _ :: _ :: _, Some _, _
         | _ :: _ :: _, _, Some _ ->
           User_error.raise
@@ -389,9 +460,9 @@ let install_uninstall ~what =
                 "Cannot specify --prefix or --libdir when installing into \
                  multiple contexts!"
             ]
-        | _ -> () );
-        let module CMap = Map.Make (Context) in
+        | _ -> ());
         let install_files_by_context =
+          let module CMap = Map.Make (Context) in
           CMap.of_list_multi install_files
           |> CMap.to_list
           |> List.map ~f:(fun (context, install_files) ->
@@ -422,14 +493,33 @@ let install_uninstall ~what =
                  in
                  (context, entries_per_package))
         in
+        let destdir =
+          if create_install_files then
+            Some (Option.value ~default:"_destdir" destdir)
+          else
+            destdir
+        in
+        let open Fiber.O in
         let (module Ops) = file_operations ~dry_run ~workspace in
         let files_deleted_in = ref Path.Set.empty in
         let+ () =
           let mandir =
             Option.map ~f:Path.of_string
-              ( match mandir with
+              (match mandir with
               | Some _ -> mandir
-              | None -> Dune_rules.Setup.mandir )
+              | None -> Dune_rules.Setup.mandir)
+          in
+          let docdir =
+            Option.map ~f:Path.of_string
+              (match docdir with
+              | Some _ -> docdir
+              | None -> Dune_rules.Setup.docdir)
+          in
+          let etcdir =
+            Option.map ~f:Path.of_string
+              (match etcdir with
+              | Some _ -> etcdir
+              | None -> Dune_rules.Setup.etcdir)
           in
           Fiber.sequential_iter install_files_by_context
             ~f:(fun (context, entries_per_package) ->
@@ -437,35 +527,69 @@ let install_uninstall ~what =
                 get_dirs context ~prefix_from_command_line
                   ~libdir_from_command_line
               in
+              let conf =
+                Dune_rules.Artifact_substitution.conf_for_install ~relocatable
+                  ~default_ocamlpath:context.default_ocamlpath
+                  ~stdlib_dir:context.stdlib_dir ~prefix ~libdir ~mandir ~docdir
+                  ~etcdir
+              in
               Fiber.sequential_iter entries_per_package
                 ~f:(fun (package, entries) ->
                   let paths =
                     Install.Section.Paths.make ~package ~destdir:prefix ?libdir
-                      ?mandir ()
+                      ?mandir ?docdir ?etcdir ()
                   in
-                  Fiber.sequential_iter entries ~f:(fun entry ->
-                      let special_file = Special_file.of_entry entry in
-                      let dst =
-                        Install.Entry.relative_installed_path entry ~paths
-                        |> interpret_destdir ~destdir
-                      in
-                      let dir = Path.parent_exn dst in
-                      if what = "install" then (
-                        Ops.remove_if_exists dst;
-                        Printf.eprintf "Installing %s\n%!"
-                          (Path.to_string_maybe_quoted dst);
-                        Ops.mkdir_p dir;
-                        let executable =
-                          Install.Section.should_set_executable_bit
-                            entry.section
+                  let+ entries =
+                    Fiber.sequential_map entries ~f:(fun entry ->
+                        let special_file = Special_file.of_entry entry in
+                        let dst =
+                          Install.Entry.relative_installed_path entry ~paths
+                          |> interpret_destdir ~destdir
                         in
-                        Ops.copy_file ~src:entry.src ~dst ~executable
-                          ~special_file ~package
-                      ) else (
-                        Ops.remove_if_exists dst;
-                        files_deleted_in := Path.Set.add !files_deleted_in dir;
-                        Fiber.return ()
-                      ))))
+                        let dir = Path.parent_exn dst in
+                        match what with
+                        | Install ->
+                          let* copy =
+                            match special_file with
+                            | _ when not create_install_files ->
+                              Fiber.return true
+                            | None ->
+                              Dune_rules.Artifact_substitution.test_file
+                                ~src:entry.src ()
+                            | Some Special_file.META
+                            | Some Special_file.Dune_package ->
+                              Fiber.return true
+                          in
+                          let msg =
+                            if create_install_files then
+                              "Copying to"
+                            else
+                              "Installing"
+                          in
+                          if copy then
+                            let* () =
+                              Ops.remove_if_exists dst;
+                              Printf.eprintf "%s %s\n%!" msg
+                                (Path.to_string_maybe_quoted dst);
+                              Ops.mkdir_p dir;
+                              let executable =
+                                Section.should_set_executable_bit entry.section
+                              in
+                              Ops.copy_file ~src:entry.src ~dst ~executable
+                                ~special_file ~package ~conf
+                            in
+                            Fiber.return (Install.Entry.set_src entry dst)
+                          else
+                            Fiber.return entry
+                        | Uninstall ->
+                          Ops.remove_if_exists dst;
+                          files_deleted_in := Path.Set.add !files_deleted_in dir;
+                          Fiber.return entry)
+                  in
+                  if create_install_files then
+                    let fn = resolve_package_install workspace package in
+                    Io.write_file (Path.source fn)
+                      (Install.gen_install_file entries)))
         in
         Path.Set.to_list !files_deleted_in
         (* This [List.rev] is to ensure we process children directories before
@@ -473,8 +597,8 @@ let install_uninstall ~what =
         |> List.rev
         |> List.iter ~f:Ops.remove_dir_if_empty)
   in
-  (term, Cmdliner.Term.info what ~doc ~man:Common.help_secs)
+  (term, Cmdliner.Term.info (cmd_what what) ~doc ~man:Common.help_secs)
 
-let install = install_uninstall ~what:"install"
+let install = install_uninstall ~what:Install
 
-let uninstall = install_uninstall ~what:"uninstall"
+let uninstall = install_uninstall ~what:Uninstall
