@@ -27,9 +27,13 @@ let pp ppf ~fields sexps =
         | _ -> false
       in
       if do_print then
+        let version =
+          Dune_lang.Syntax.greatest_supported_version Dune_engine.Stanza.syntax
+        in
         Dune_lang.Ast.add_loc sexp ~loc:Loc.none
         |> Dune_lang.Cst.concrete |> List.singleton
-        |> Format.fprintf ppf "%a@?" Dune_engine.Format_dune_lang.pp_top_sexps)
+        |> Dune_engine.Format_dune_lang.pp_top_sexps ~version
+        |> Format.fprintf ppf "%a@?" Pp.to_fmt)
 
 let term =
   let+ common = Common.term
@@ -50,7 +54,7 @@ let term =
       let checked = Util.check_path setup.workspace.contexts dir in
       let request =
         Build.all
-          ( match checked with
+          (match checked with
           | In_build_dir (ctx, _) ->
             let sctx =
               Dune_engine.Context_name.Map.find_exn setup.scontexts ctx.name
@@ -69,7 +73,7 @@ let term =
               [ Pp.text "Environment is not defined for external paths" ]
           | In_install_dir _ ->
             User_error.raise
-              [ Pp.text "Environment is not defined in install dirs" ] )
+              [ Pp.text "Environment is not defined in install dirs" ])
       in
       Build_system.do_build ~request >>| function
       | [ (_, env) ] -> Format.printf "%a" (pp ~fields) env
