@@ -4,11 +4,11 @@ type t =
   | File of Path.t
   | Alias of Alias.t
 
-val request : t list -> unit Dune.Build.t
+val request : t list -> unit Dune_engine.Build.t
 
 val resolve_target :
      Common.t
-  -> setup:Dune.Main.build_system
+  -> setup:Dune_rules.Main.build_system
   -> Arg.Dep.t
   -> (t list, Arg.Dep.t * User_message.Style.t Pp.t list) result
 
@@ -18,9 +18,9 @@ type resolve_input =
 
 val resolve_targets_mixed :
      Common.t
-  -> Dune.Main.build_system
+  -> Dune_rules.Main.build_system
   -> resolve_input list
   -> (t list, Arg.Dep.t * User_message.Style.t Pp.t list) result list
 
 val resolve_targets_exn :
-  Common.t -> Dune.Main.build_system -> Arg.Dep.t list -> t list
+  Common.t -> Dune_rules.Main.build_system -> Arg.Dep.t list -> t list
