@@ -47,7 +47,7 @@ let term =
       let requires =
         Dune_rules.Lib.closure ~linking:true libs |> Result.ok_exn
       in
-      let include_paths = Dune_rules.Lib.L.include_paths requires in
+      let include_paths = Dune_rules.Lib.L.toplevel_include_paths requires in
       let files = link_deps requires in
       let* () = do_build (List.map files ~f:(fun f -> Target.File f)) in
       let files_to_load =
