@@ -28,7 +28,8 @@ let fake_gc_stat =
   ; compactions = 0
   ; top_heap_words = 0
   ; stack_size = 0
-  } [@ocaml.warning "-23"]
+  }
+  [@ocaml.warning "-23"]
 
 (* all fiels of record used *)
 
