@@ -112,7 +112,7 @@ end = struct
           |> Path.of_string |> Path.as_in_source_tree
         with
         | Some p -> Anonymous p
-        | None -> invalid s )
+        | None -> invalid s)
       | _ when validate s -> Named s
       | _ -> invalid s
 end
@@ -160,8 +160,8 @@ type t =
   ; implicit_transitive_deps : bool
   ; wrapped_executables : bool
   ; dune_version : Dune_lang.Syntax.Version.t
-  ; allow_approx_merlin : bool
   ; generate_opam_files : bool
+  ; use_standard_c_and_cxx_flags : bool option
   ; file_key : File_key.t
   ; dialects : Dialect.DB.t
   ; explicit_js_mode : bool
@@ -190,10 +190,10 @@ let file_key t = t.file_key
 
 let implicit_transitive_deps t = t.implicit_transitive_deps
 
-let allow_approx_merlin t = t.allow_approx_merlin
-
 let generate_opam_files t = t.generate_opam_files
 
+let use_standard_c_and_cxx_flags t = t.use_standard_c_and_cxx_flags
+
 let dialects t = t.dialects
 
 let explicit_js_mode t = t.explicit_js_mode
@@ -211,8 +211,8 @@ let to_dyn
     ; implicit_transitive_deps
     ; wrapped_executables
     ; dune_version
-    ; allow_approx_merlin
     ; generate_opam_files
+    ; use_standard_c_and_cxx_flags
     ; file_key
     ; dialects
     ; explicit_js_mode
@@ -233,12 +233,12 @@ let to_dyn
     ; ("implicit_transitive_deps", bool implicit_transitive_deps)
     ; ("wrapped_executables", bool wrapped_executables)
     ; ("dune_version", Dune_lang.Syntax.Version.to_dyn dune_version)
-    ; ("allow_approx_merlin", bool allow_approx_merlin)
     ; ("generate_opam_files", bool generate_opam_files)
+    ; ("use_standard_c_and_cxx_flags", option bool use_standard_c_and_cxx_flags)
     ; ("file_key", string file_key)
     ; ("dialects", Dialect.DB.to_dyn dialects)
     ; ("explicit_js_mode", bool explicit_js_mode)
-    ; ("format_config", (option Format_config.to_dyn) format_config)
+    ; ("format_config", option Format_config.to_dyn format_config)
     ; ("strict_package_deps", bool strict_package_deps)
     ; ("cram", bool cram)
     ]
@@ -349,8 +349,8 @@ module Extension = struct
         -> Univ_map.t * Stanza.Parser.t list
     }
 
-  (* CR-soon amokhov: convert this mutable table to a memoized function, which
-     depends on the contents of dune files that declare extensions. *)
+  (* CR-someday amokhov: convert this mutable table to a memoized function,
+     which depends on the contents of dune files that declare extensions. *)
   let extensions = Table.create (module String) 32
 
   let register syntax stanzas arg_to_dyn =
@@ -433,16 +433,16 @@ module Extension = struct
                         if not !dune_project_edited then (
                           dune_project_edited := true;
                           ignore
-                            ( Project_file_edit.append project_file
-                                (List
-                                   [ Dune_lang.atom "using"
-                                   ; Dune_lang.atom
-                                       (Dune_lang.Syntax.name e.syntax)
-                                   ; Dune_lang.atom
-                                       (Dune_lang.Syntax.Version.to_string
-                                          version)
-                                   ])
-                              : created_or_already_exist )
+                            (Project_file_edit.append project_file
+                               (List
+                                  [ Dune_lang.atom "using"
+                                  ; Dune_lang.atom
+                                      (Dune_lang.Syntax.name e.syntax)
+                                  ; Dune_lang.atom
+                                      (Dune_lang.Syntax.Version.to_string
+                                         version)
+                                  ])
+                              : created_or_already_exist)
                         );
                         p ))
                 in
@@ -450,7 +450,7 @@ module Extension = struct
               in
               Enabled
                 { extension = Packed e; version; loc = Loc.none; parse_args }
-              :: acc ))
+              :: acc))
 end
 
 let interpret_lang_and_extensions ~(lang : Lang.Instance.t) ~explicit_extensions
@@ -504,9 +504,10 @@ let interpret_lang_and_extensions ~(lang : Lang.Instance.t) ~explicit_extensions
     in
     let extension_args, extension_stanzas =
       List.fold_left extensions ~init:(Univ_map.empty, [])
-        ~f:(fun (args_acc, stanzas_acc)
-                ((ext : Extension.automatic), is_explicit)
-                ->
+        ~f:(fun
+             (args_acc, stanzas_acc)
+             ((ext : Extension.automatic), is_explicit)
+           ->
           match ext with
           | Disabled _ -> (args_acc, stanzas_acc)
           | Enabled instance ->
@@ -561,7 +562,8 @@ let format_extension_key =
 let format_config t =
   let ext = find_extension_args t format_extension_key in
   let dune_lang = t.format_config in
-  Format_config.of_config ~ext ~dune_lang
+  let version = t.dune_version in
+  Format_config.of_config ~ext ~dune_lang ~version
 
 let default_name ~dir ~(packages : Package.t Package.Name.Map.t) =
   match Package.Name.Map.min_binding packages with
@@ -575,7 +577,7 @@ let default_name ~dir ~(packages : Package.t Package.Name.Map.t) =
          rejected as a valid Dune project name. It would be better to make the
          set of allowed package names and the set of project names coincide. *)
       User_error.raise ~loc:pkg.loc
-        [ Pp.textf "%S is not a valid Dune project name." name ] )
+        [ Pp.textf "%S is not a valid Dune project name." name ])
 
 let infer ~dir packages =
   let lang = get_dune_lang () in
@@ -607,8 +609,12 @@ let infer ~dir packages =
   ; extension_args
   ; parsing_context
   ; dune_version = lang.version
-  ; allow_approx_merlin = true
   ; generate_opam_files = false
+  ; use_standard_c_and_cxx_flags =
+      (if lang.version < (3, 0) then
+        None
+      else
+        Some true)
   ; file_key
   ; dialects = Dialect.DB.builtin
   ; explicit_js_mode
@@ -645,7 +651,7 @@ end
 
 let anonymous ~dir = infer ~dir Package.Name.Map.empty
 
-let parse ~dir ~lang ~opam_packages ~file =
+let parse ~dir ~lang ~opam_packages ~file ~dir_status =
   fields
     (let+ name = field_o "name" Name.decode
      and+ version = field_o "version" string
@@ -667,21 +673,43 @@ let parse ~dir ~lang ~opam_packages ~file =
      and+ wrapped_executables =
        field_o_b "wrapped_executables"
          ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 11))
-     and+ allow_approx_merlin =
-       field_o_b "allow_approximate_merlin"
-         ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 9))
+     and+ _allow_approx_merlin =
+       (* TODO DUNE3 remove this field from parsing *)
+       let+ loc = loc
+       and+ f =
+         field_o_b "allow_approximate_merlin"
+           ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 9))
+       in
+       let vendored =
+         match dir_status with
+         | Sub_dirs.Status.Vendored -> true
+         | _ -> false
+       in
+       if
+         Option.is_some f
+         && Dune_lang.Syntax.Version.Infix.(lang.version >= (2, 8))
+         && not vendored
+       then
+         Dune_lang.Syntax.Warning.deprecated_in
+           ~extra_info:
+             "It is useless since the Merlin configurations are not ambiguous \
+              anymore."
+           loc lang.syntax (2, 8) ~what:"This field"
      and+ () = Dune_lang.Versioned_file.no_more_lang
      and+ generate_opam_files =
        field_o_b "generate_opam_files"
          ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 10))
+     and+ use_standard_c_and_cxx_flags =
+       field_o_b "use_standard_c_and_cxx_flags"
+         ~check:(Dune_lang.Syntax.since Stanza.syntax (2, 8))
      and+ dialects =
        multi_field "dialect"
-         ( Dune_lang.Syntax.since Stanza.syntax (1, 11)
-         >>> located Dialect.decode )
+         (Dune_lang.Syntax.since Stanza.syntax (1, 11)
+         >>> located Dialect.decode)
      and+ explicit_js_mode =
        field_o_b "explicit_js_mode"
          ~check:(Dune_lang.Syntax.since Stanza.syntax (1, 11))
-     and+ format_config = Format_config.field
+     and+ format_config = Format_config.field ~since:(2, 0)
      and+ strict_package_deps =
        field_o_b "strict_package_deps"
          ~check:(Dune_lang.Syntax.since Stanza.syntax (2, 3))
@@ -692,16 +720,16 @@ let parse ~dir ~lang ~opam_packages ~file =
        if List.is_empty packages then
          Package.Name.Map.map opam_packages ~f:(fun (_loc, p) -> Lazy.force p)
        else (
-         ( match (packages, name) with
+         (match (packages, name) with
          | [ p ], Some (Named name) ->
-           if Package.Name.to_string p.name <> name then
+           if Package.Name.to_string (Package.name p) <> name then
              User_error.raise ~loc:p.loc
                [ Pp.textf
                    "when a single package is defined, it must have the same \
                     name as the project name: %s"
                    name
                ]
-         | _, _ -> () );
+         | _, _ -> ());
          let package_defined_twice name loc1 loc2 =
            User_error.raise
              [ Pp.textf "Package name %s is defined twice:"
@@ -717,18 +745,19 @@ let parse ~dir ~lang ~opam_packages ~file =
                  ~f:package_defined_twice)
          in
          List.iter packages ~f:(fun p ->
-             match
-               Package.Name.Map.find deprecated_package_names p.Package.name
-             with
+             let name = Package.name p in
+             match Package.Name.Map.find deprecated_package_names name with
              | None -> ()
-             | Some loc -> package_defined_twice p.Package.name loc p.loc);
+             | Some loc -> package_defined_twice name loc p.loc);
          match
-           Package.Name.Map.of_list_map packages ~f:(fun p -> (p.name, p))
+           Package.Name.Map.of_list_map packages ~f:(fun p ->
+               (Package.name p, p))
          with
          | Error (_, _, p) ->
+           let name = Package.name p in
            User_error.raise ~loc:p.loc
              [ Pp.textf "package %s is already defined"
-                 (Package.Name.to_string p.name)
+                 (Package.Name.to_string name)
              ]
          | Ok packages ->
            Package.Name.Map.merge packages opam_packages
@@ -781,15 +810,18 @@ let parse ~dir ~lang ~opam_packages ~file =
          ~default:(strict_package_deps_default ~lang)
      in
      let dune_version = lang.version in
-     let allow_approx_merlin =
-       Option.value ~default:(dune_version < (1, 9)) allow_approx_merlin
-     in
      let explicit_js_mode =
        Option.value explicit_js_mode ~default:(explicit_js_mode_default ~lang)
      in
      let generate_opam_files =
        Option.value ~default:false generate_opam_files
      in
+     let use_standard_c_and_cxx_flags =
+       match use_standard_c_and_cxx_flags with
+       | None when dune_version >= (3, 0) -> Some false
+       | None -> None
+       | some -> some
+     in
      let cram =
        match cram with
        | None -> false
@@ -803,6 +835,23 @@ let parse ~dir ~lang ~opam_packages ~file =
            Dialect.DB.add dialects ~loc dialect)
          ~init:Dialect.DB.builtin dialects
      in
+     let () =
+       match name with
+       | Named _ -> ()
+       | Anonymous _ ->
+         if
+           dune_version >= (2, 8)
+           && generate_opam_files
+           && dir_status = Sub_dirs.Status.Normal
+         then
+           let loc = Loc.in_file (Path.source file) in
+           User_warning.emit ~loc
+             [ Pp.text
+                 "Project name is not specified. Add a (name <project-name>) \
+                  field to your dune-project file to make sure that $ dune \
+                  subst works in release or pinned builds"
+             ]
+     in
      { name
      ; file_key
      ; root
@@ -816,8 +865,8 @@ let parse ~dir ~lang ~opam_packages ~file =
      ; implicit_transitive_deps
      ; wrapped_executables
      ; dune_version
-     ; allow_approx_merlin
      ; generate_opam_files
+     ; use_standard_c_and_cxx_flags
      ; dialects
      ; explicit_js_mode
      ; format_config
@@ -825,12 +874,12 @@ let parse ~dir ~lang ~opam_packages ~file =
      ; cram
      })
 
-let load_dune_project ~dir opam_packages =
+let load_dune_project ~dir opam_packages ~dir_status =
   let file = Path.Source.relative dir filename in
   load_exn (Path.source file) ~f:(fun lang ->
-      parse ~dir ~lang ~opam_packages ~file)
+      parse ~dir ~lang ~opam_packages ~file ~dir_status)
 
-let load ~dir ~files ~infer_from_opam_files =
+let load ~dir ~files ~infer_from_opam_files ~dir_status =
   let opam_packages =
     String.Set.fold files ~init:[] ~f:(fun fn acc ->
         match Package.Name.of_opam_file_basename fn with
@@ -846,7 +895,7 @@ let load ~dir ~files ~infer_from_opam_files =
     |> Package.Name.Map.of_list_exn
   in
   if String.Set.mem files filename then
-    Some (load_dune_project ~dir opam_packages)
+    Some (load_dune_project ~dir opam_packages ~dir_status)
   else if
     Path.Source.is_root dir
     || (infer_from_opam_files && not (Package.Name.Map.is_empty opam_packages))
@@ -866,8 +915,11 @@ let wrapped_executables t = t.wrapped_executables
 
 let () =
   let open Dune_lang.Decoder in
-  Extension.register_simple Action_plugin.syntax (return [])
+  Extension.register_simple Action_plugin.syntax (return []);
+  Extension.register_simple Section.dune_site_syntax (return [])
 
 let strict_package_deps t = t.strict_package_deps
 
 let cram t = t.cram
+
+let info t = t.info
