@@ -139,8 +139,19 @@ end = struct
     || String.is_suffix fn ~suffix:".swp"
     || String.is_suffix fn ~suffix:"~"
 
+  (* Returns [true] for special files such as character devices of sockets; see
+     #3124 for more on issues caused by special devices *)
+  let is_special (st_kind : Unix.file_kind) =
+    match st_kind with
+    | S_CHR
+    | S_BLK
+    | S_FIFO
+    | S_SOCK ->
+      true
+    | _ -> false
+
   let of_source_path path =
-    match Path.readdir_unsorted (Path.source path) with
+    match Path.readdir_unsorted_with_kinds (Path.source path) with
     | Error unix_error ->
       User_warning.emit
         [ Pp.textf "Unable to read directory %s. Ignoring."
@@ -157,20 +168,24 @@ end = struct
       Error unix_error
     | Ok unsorted_contents ->
       let files, dirs =
-        List.filter_partition_map unsorted_contents ~f:(fun fn ->
+        List.filter_partition_map unsorted_contents ~f:(fun (fn, kind) ->
             let path = Path.Source.relative path fn in
             if Path.Source.is_in_build_dir path then
               Skip
             else
               let is_directory, file =
-                match Path.stat (Path.source path) with
-                | exception _ -> (false, File.dummy)
-                | { st_kind = S_DIR; _ } as st -> (true, File.of_stats st)
+                match kind with
+                | S_DIR -> (true, File.of_source_path path)
+                | S_LNK -> (
+                  match Path.stat (Path.source path) with
+                  | exception _ -> (false, File.dummy)
+                  | { st_kind = S_DIR; _ } as st -> (true, File.of_stats st)
+                  | _ -> (false, File.dummy))
                 | _ -> (false, File.dummy)
               in
               if is_directory then
                 Right (fn, path, file)
-              else if is_temp_file fn then
+              else if is_temp_file fn || is_special kind then
                 Skip
               else
                 Left fn)
@@ -412,10 +427,10 @@ end = struct
       | Ignored -> None
       | Status status ->
         Some
-          ( match (parent_status, status) with
+          (match (parent_status, status) with
           | Data_only, _ -> Data_only
           | Vendored, Normal -> Vendored
-          | _, _ -> status )
+          | _, _ -> status)
 
     let make_subdir ~dir_status ~virtual_ path =
       let sub_dir_as_t = find_dir_raw path in
@@ -499,8 +514,8 @@ end = struct
       None
     else (
       ignore
-        ( Dune_project.ensure_project_file_exists project
-          : Dune_project.created_or_already_exist );
+        (Dune_project.ensure_project_file_exists project
+          : Dune_project.created_or_already_exist);
       let file = Path.Source.relative path Dune_file.fname in
       Some (Dune_file.load file ~file_exists ~project ~from_parent)
     )
@@ -549,7 +564,7 @@ end = struct
     let project =
       match
         Dune_project.load ~dir:path ~files:readdir.files
-          ~infer_from_opam_files:true
+          ~infer_from_opam_files:true ~dir_status
       with
       | None -> Dune_project.anonymous ~dir:path
       | Some p -> p
@@ -580,7 +595,6 @@ end = struct
           if
             Dune_project.cram parent_dir.project
             && Cram_test.is_cram_suffix basename
-            && status = Normal
           then
             Sub_dirs.Status.Data_only
           else
@@ -604,7 +618,8 @@ end = struct
         else
           Option.value
             (Dune_project.load ~dir:path ~files:readdir.files
-               ~infer_from_opam_files:settings.recognize_jbuilder_projects)
+               ~infer_from_opam_files:settings.recognize_jbuilder_projects
+               ~dir_status)
             ~default:parent_dir.project
       in
       let vcs = get_vcs ~default:parent_dir.vcs ~readdir ~path in
@@ -650,7 +665,7 @@ let rec nearest_dir t = function
     | Some _ ->
       let path = Path.Source.relative (Dir0.path t) comp in
       let dir = Option.value_exn (find_dir path) in
-      nearest_dir dir components )
+      nearest_dir dir components)
 
 let nearest_dir path =
   let components = Path.Source.explode path in
@@ -737,10 +752,10 @@ module Dir = struct
                    None
                  else
                    Some
-                     ( if String.Set.mem files fname then
+                     (if String.Set.mem files fname then
                        Ok test
                      else
-                       Error (Missing_run_t test) ))
+                       Error (Missing_run_t test)))
       in
       file_tests @ dir_tests
 end
@@ -767,3 +782,8 @@ let find_dir_specified_on_command_line ~dir =
       [ Pp.textf "Don't know about directory %s specified on the command line!"
           (Path.Source.to_string_maybe_quoted dir)
       ]
+
+let is_vendored dir =
+  match find_dir dir with
+  | None -> false
+  | Some d -> Dir.status d = Vendored
