@@ -8,11 +8,20 @@ type dune_file =
 val parse_file : Path.t option -> dune_file
 
 (** Write the formatted concrete syntax to the file at [path] *)
-val write_file : path:Path.t -> Dune_lang.Cst.t list -> unit
+val write_file :
+     version:Dune_lang.Syntax.Version.t
+  -> path:Path.t
+  -> Dune_lang.Cst.t list
+  -> unit
 
 (** Reformat a dune file. [None] in [input] corresponds to stdin. [None] in
     [output] corresponds to stdout. *)
-val format_file : input:Path.t option -> output:Path.t option -> unit
+val format_file :
+     version:Dune_lang.Syntax.Version.t
+  -> input:Path.t option
+  -> output:Path.t option
+  -> unit
 
 (** Pretty-print a list of toplevel s-expressions *)
-val pp_top_sexps : Format.formatter -> Dune_lang.Cst.t list -> unit
+val pp_top_sexps :
+  version:Dune_lang.Syntax.Version.t -> Dune_lang.Cst.t list -> _ Pp.t
