@@ -107,7 +107,7 @@ end = struct
 
   let cond = Condition.create ()
 
-  (* CR-soon amokhov: The way we handle "ignored files" using this mutable table
+  (* CR-someday amokhov: The way we handle "ignored files" using a mutable table
      is fragile and also wrong. We use [ignored_files] for the [(mode promote)]
      feature: if a file is promoted, we call [ignore_next_file_change_event] so
      that the upcoming file-change event does not invalidate the current build.
@@ -125,20 +125,20 @@ end = struct
 
   let available () =
     not
-      ( List.is_empty !files_changed
+      (List.is_empty !files_changed
       && Queue.is_empty jobs_completed
       && Signal.Set.is_empty !signals
-      && Queue.is_empty dedup_pending )
+      && Queue.is_empty dedup_pending)
 
   let dedup () =
     match Queue.pop dedup_pending with
     | None -> false
     | Some pending ->
       let (module Caching : Cache.Caching), (file : Cache.File.t) = pending in
-      ( match Cached_digest.peek_file (Path.build file.path) with
+      (match Cached_digest.peek_file (Path.build file.path) with
       | None -> ()
       | Some d when not (Digest.equal d file.digest) -> ()
-      | _ -> Caching.Cache.deduplicate Caching.cache file );
+      | _ -> Caching.Cache.deduplicate Caching.cache file);
       true
 
   let rec flush_dedup () = if dedup () then flush_dedup ()
@@ -162,6 +162,7 @@ end = struct
           | [] ->
             let job, status = Queue.pop_exn jobs_completed in
             decr pending_jobs;
+            assert (!pending_jobs >= 0);
             Job_completed (job, status)
           | fns ->
             files_changed := [];
@@ -178,7 +179,7 @@ end = struct
             if only_ignored_files then
               loop ()
             else
-              Files_changed )
+              Files_changed)
     in
     let ev = loop () in
     Mutex.unlock mutex;
@@ -294,13 +295,13 @@ end = struct
          | None ->
            User_error.raise
              [ Pp.text
-                 ( if Sys.linux then
+                 (if Sys.linux then
                    "Please install inotifywait to enable watch mode. If \
                     inotifywait is unavailable, fswatch may also be used but \
                     will result in a worse experience."
                  else
-                   "Please install fswatch to enable watch mode." )
-             ] ))
+                   "Please install fswatch to enable watch mode.")
+             ]))
 
   let buffering_time = 0.5 (* seconds *)
 
@@ -321,11 +322,11 @@ end = struct
     for i = 0 to buffer.size - 1 do
       let c = Bytes.get buffer.data i in
       if c = '\n' || c = '\r' then (
-        ( if !line_start < i then
+        (if !line_start < i then
           let line =
             Bytes.sub_string buffer.data ~pos:!line_start ~len:(i - !line_start)
           in
-          lines := line :: !lines );
+          lines := line :: !lines);
         line_start := i + 1
       )
     done;
@@ -417,8 +418,6 @@ end = struct
 
     (* This mutable table is safe: it does not interact with the state we track
        in the build system. *)
-    (* Invariant: [!running_count] is equal to the number of [Running _] values
-       in [table]. *)
     let table = Table.create (module Pid) 128
 
     let running_count = ref 0
@@ -461,7 +460,8 @@ end = struct
   let killall signal =
     Mutex.lock mutex;
     Process_table.iter ~f:(fun job ->
-        try Unix.kill (Pid.to_int job.pid) signal with Unix.Unix_error _ -> ());
+        try Unix.kill (Pid.to_int job.pid) signal with
+        | Unix.Unix_error _ -> ());
     Mutex.unlock mutex
 
   exception Finished of job * Unix.process_status
@@ -472,7 +472,8 @@ end = struct
           let pid, status = Unix.waitpid [ WNOHANG ] (Pid.to_int job.pid) in
           if pid <> 0 then raise_notrace (Finished (job, status)));
       false
-    with Finished (job, status) ->
+    with
+    | Finished (job, status) ->
       (* We need to do the [Unix.waitpid] and remove the process while holding
          the lock, otherwise the pid might be reused in between. *)
       Process_table.remove ~pid:job.pid status;
@@ -481,7 +482,7 @@ end = struct
   let wait_win32 () =
     while not (wait_nonblocking_win32 ()) do
       Mutex.unlock mutex;
-      ignore (Unix.select [] [] [] 0.001);
+      Thread.delay 0.001;
       Mutex.lock mutex
     done
 
@@ -570,9 +571,19 @@ end = struct
   let init () = Lazy.force init
 end
 
+type status =
+  (* Waiting for file changes to start a new a build *)
+  | Waiting_for_file_changes of unit Fiber.Ivar.t
+  (* Running a build *)
+  | Building
+  (* Cancellation requested. Build jobs are immediately rejected in this state *)
+  | Restarting_build
+
 type t =
   { original_cwd : string
-  ; mutable job_throttle : Fiber.Throttle.t
+  ; polling : bool
+  ; mutable status : status
+  ; job_throttle : Fiber.Throttle.t
   }
 
 let t_var : t Fiber.Var.t = Fiber.Var.create ()
@@ -580,15 +591,24 @@ let t_var : t Fiber.Var.t = Fiber.Var.create ()
 let with_chdir ~dir ~f =
   let t = Fiber.Var.get_exn t_var in
   Sys.chdir (Path.to_string dir);
-  protectx () ~finally:(fun () -> Sys.chdir t.original_cwd) ~f
+  protect ~finally:(fun () -> Sys.chdir t.original_cwd) ~f
 
 let set_concurrency n =
   let t = Fiber.Var.get_exn t_var in
   Fiber.Throttle.resize t.job_throttle n
 
+exception Cancel_build
+
 let with_job_slot f =
   let t = Fiber.Var.get_exn t_var in
-  Fiber.Throttle.run t.job_throttle ~f
+  Fiber.Throttle.run t.job_throttle ~f:(fun () ->
+      match t.status with
+      | Restarting_build -> raise Cancel_build
+      | Building -> f ()
+      | Waiting_for_file_changes _ ->
+        (* At this stage, we're not running a build, so we shouldn't be running
+           tasks here. *)
+        assert false)
 
 let wait_for_process pid =
   let ivar = Fiber.Ivar.create () in
@@ -605,10 +625,7 @@ type saw_signal =
   | Ok
   | Got_signal
 
-let kill_and_wait_for_all_processes t () =
-  (* Re-create the throttle so that we don't keep old ivars from one run to the
-     next *)
-  t.job_throttle <- Fiber.Throttle.create (Fiber.Throttle.size t.job_throttle);
+let kill_and_wait_for_all_processes () =
   Process_watcher.killall Sys.sigkill;
   let saw_signal = ref Ok in
   while Event.pending_jobs () > 0 do
@@ -620,7 +637,7 @@ let kill_and_wait_for_all_processes t () =
   done;
   !saw_signal
 
-let prepare ?(config = Config.default) () =
+let prepare ?(config = Config.default) ~polling () =
   Log.info
     [ Pp.textf "Workspace root: %s"
         (Path.to_absolute_filename Path.root |> String.maybe_quoted)
@@ -632,7 +649,9 @@ let prepare ?(config = Config.default) () =
   let cwd = Sys.getcwd () in
   if cwd <> initial_cwd && not !Clflags.no_print_directory then
     Printf.eprintf "Entering directory '%s'\n%!"
-      ( if Config.inside_dune then
+      (match Config.inside_dune with
+      | false -> cwd
+      | true -> (
         let descendant_simple p ~of_ =
           match String.drop_prefix p ~prefix:of_ with
           | None
@@ -652,16 +671,16 @@ let prepare ?(config = Config.default) () =
               else
                 loop (Filename.concat acc "..") (Filename.dirname dir)
             in
-            loop ".." (Filename.dirname s) )
-      else
-        cwd );
+            loop ".." (Filename.dirname s))));
   let t =
     { original_cwd = cwd
+    ; status = Building
     ; job_throttle =
         Fiber.Throttle.create
-          ( match config.concurrency with
+          (match config.concurrency with
           | Auto -> 1
-          | Fixed n -> n )
+          | Fixed n -> n)
+    ; polling
     }
   in
   t
@@ -669,7 +688,6 @@ let prepare ?(config = Config.default) () =
 module Run_once : sig
   type run_error =
     | Got_signal
-    | Files_changed
     | Never
     | Exn of Exn_with_backtrace.t
 
@@ -678,21 +696,51 @@ module Run_once : sig
 end = struct
   type run_error =
     | Got_signal
-    | Files_changed
     | Never
     | Exn of Exn_with_backtrace.t
 
   exception Abort of run_error
 
-  let iter () =
-    let count = Event.pending_jobs () in
-    if count = 0 then
+  (** This function is the heart of the scheduler. It makes progress in
+      executing fibers by doing the following:
+
+      - notifying completed jobs
+      - starting cancellations
+      - terminating the scheduler on signals *)
+  let rec iter (t : t) =
+    if
+      (match t.status with
+      | Waiting_for_file_changes _ ->
+        (* In polling mode, there are no pending jobs while we are waiting for
+           file changes *)
+        false
+      | _ -> true)
+      && Event.pending_jobs () = 0
+    then
       raise (Abort Never)
     else (
       Console.Status_line.refresh ();
       match Event.next () with
       | Job_completed (job, status) -> Fiber.Fill (job.ivar, status)
-      | Files_changed -> raise (Abort Files_changed)
+      | Files_changed -> (
+        match t.status with
+        | Restarting_build ->
+          (* We're already cancelling build, so file change events don't matter *)
+          iter t
+        | Building ->
+          let status_line =
+            Some
+              (Pp.seq
+                 (* XXX Why do we print "Had errors"? The user simply edited a
+                    file *)
+                 (Pp.tag User_message.Style.Error (Pp.verbatim "Had errors"))
+                 (Pp.verbatim ", killing current build..."))
+          in
+          Console.Status_line.set (Fun.const status_line);
+          t.status <- Restarting_build;
+          Process_watcher.killall Sys.sigkill;
+          iter t
+        | Waiting_for_file_changes ivar -> Fill (ivar, ()))
       | Signal signal ->
         got_signal signal;
         raise (Abort Got_signal)
@@ -702,45 +750,28 @@ end = struct
     let fiber =
       Fiber.Var.set t_var t (fun () -> Fiber.with_error_handler f ~on_error)
     in
-    Console.Status_line.set (Fun.const None);
-    match Fiber.run fiber ~iter with
+    match Fiber.run fiber ~iter:(fun () -> iter t) with
     | res ->
-      assert (Event.pending_jobs () = 0);
+      assert ((not t.polling) && Event.pending_jobs () = 0);
       Ok res
     | exception Abort err -> Error err
     | exception exn -> Error (Exn (Exn_with_backtrace.capture exn))
 
   let run_and_cleanup t f =
     let res = run t f in
-    ( match res with
-    | Error Files_changed ->
-      Console.Status_line.set (fun () ->
-          Some
-            (Pp.seq
-               (Pp.tag User_message.Style.Error (Pp.verbatim "Had errors"))
-               (Pp.verbatim ", killing current build...")))
-    | _ -> () );
-    match kill_and_wait_for_all_processes t () with
+    Console.Status_line.set (Fun.const None);
+    match kill_and_wait_for_all_processes () with
     | Got_signal -> Error Got_signal
     | Ok -> res
 end
 
 let go ?config f =
-  let t = prepare ?config () in
+  let t = prepare ?config ~polling:false () in
   let res = Run_once.run_and_cleanup t f in
   match res with
   | Error (Exn exn) -> Exn_with_backtrace.reraise exn
   | Ok res -> res
   | Error (Got_signal | Never) -> raise Dune_util.Report_error.Already_reported
-  | Error Files_changed ->
-    Code_error.raise
-      "Scheduler.go: files changed even though we're running without \
-       filesystem watcher"
-      []
-
-type exit_or_continue =
-  | Exit
-  | Continue
 
 let maybe_clear_screen ~config =
   match
@@ -759,45 +790,60 @@ let maybe_clear_screen ~config =
          ])
 
 let poll ?config ~once ~finally () =
-  let t = prepare ?config () in
+  let t = prepare ?config ~polling:true () in
   let watcher = File_watcher.create () in
-  let block_waiting_for_changes () =
-    match Event.next () with
-    | Job_completed _ -> assert false
-    | Files_changed -> Continue
-    | Signal signal ->
-      got_signal signal;
-      Exit
+  let rec loop () : Nothing.t Fiber.t =
+    t.status <- Building;
+    let open Fiber.O in
+    let* res =
+      let+ res =
+        let on_error exn () =
+          match t.status with
+          | Building -> Report_error.report exn
+          | Restarting_build -> ()
+          | Waiting_for_file_changes _ ->
+            (* We are inside a build, so we aren't waiting for a file change
+               event *)
+            assert false
+        in
+        Fiber.fold_errors ~init:() ~on_error once
+      in
+      finally ();
+      res
+    in
+    match t.status with
+    | Waiting_for_file_changes _ ->
+      (* We just finished a build, so there's no way this was set *)
+      assert false
+    | Restarting_build -> loop ()
+    | Building ->
+      let message =
+        match res with
+        | Ok () -> Pp.tag User_message.Style.Success (Pp.verbatim "Success")
+        | Error _ -> Pp.tag User_message.Style.Error (Pp.verbatim "Had errors")
+      in
+      Console.Status_line.set
+        (Fun.const
+           (Some
+              (Pp.seq message
+                 (Pp.verbatim ", waiting for filesystem changes..."))));
+      let ivar = Fiber.Ivar.create () in
+      t.status <- Waiting_for_file_changes ivar;
+      let* () = Fiber.Ivar.read ivar in
+      maybe_clear_screen ~config;
+      loop ()
   in
-  let wait msg =
-    Console.Status_line.set_temporarily
-      (fun () ->
-        Some (Pp.seq msg (Pp.verbatim ", waiting for filesystem changes...")))
-      (fun () ->
-        let res = block_waiting_for_changes () in
-        maybe_clear_screen ~config;
-        res)
-  in
-  let rec loop () =
-    let res = Run_once.run_and_cleanup t once in
-    finally ();
-    match res with
-    | Ok () ->
-      wait (Pp.tag User_message.Style.Success (Pp.verbatim "Success"))
-      |> after_wait
-    | Error Got_signal -> (Dune_util.Report_error.Already_reported, None)
-    | Error Never ->
-      wait (Pp.tag User_message.Style.Error (Pp.verbatim "Had errors"))
-      |> after_wait
-    | Error Files_changed -> loop ()
+  let exn, bt =
+    match Run_once.run_and_cleanup t loop with
+    | Ok (_ : Nothing.t) ->
+      (* Polling mode is an infinite loop. We aren't going to terminate *)
+      .
+    | Error (Got_signal | Never) ->
+      (Dune_util.Report_error.Already_reported, None)
     | Error (Exn exn_with_bt) -> (exn_with_bt.exn, Some exn_with_bt.backtrace)
-  and after_wait = function
-    | Exit -> (Dune_util.Report_error.Already_reported, None)
-    | Continue -> loop ()
   in
-  let exn, bt = loop () in
   ignore (wait_for_process (File_watcher.pid watcher) : _ Fiber.t);
-  ignore (kill_and_wait_for_all_processes t () : saw_signal);
+  ignore (kill_and_wait_for_all_processes () : saw_signal);
   match bt with
   | None -> Exn.raise exn
   | Some bt -> Exn.raise_with_backtrace exn bt
