@@ -10,6 +10,10 @@ val of_string : string -> t
 
 val to_string : t -> string
 
+(** [parse s] is [Some (a:t)] if [s] can be a valid atom according to [is_valid]
+    otherwise it is [None] *)
+val parse : string -> t option
+
 val print : t -> string
 
 val of_int : int -> t
