@@ -187,7 +187,7 @@ module Partial = struct
       | Unexpanded tmpl ->
         let path = E.path ~expander x in
         check_mkdir (String_with_vars.loc tmpl) path;
-        Mkdir path )
+        Mkdir path)
     | Digest_files x -> Digest_files (List.map x ~f:(E.path ~expander))
     | Diff { optional; file1; file2; mode } ->
       Diff
@@ -203,8 +203,8 @@ module Partial = struct
         , E.target ~expander target )
     | No_infer t -> No_infer (expand t ~expander)
     | Pipe (outputs, l) -> Pipe (outputs, List.map l ~f:(expand ~expander))
-    | Format_dune_file (src, dst) ->
-      Format_dune_file (E.path ~expander src, E.target ~expander dst)
+    | Format_dune_file (ver, src, dst) ->
+      Format_dune_file (ver, E.path ~expander src, E.target ~expander dst)
     | Cram script -> Cram (E.path ~expander script)
 end
 
@@ -265,7 +265,7 @@ let rec partial_expand t ~expander : Partial.t =
       User_error.raise ~loc
         [ Pp.text "This directory cannot be evaluated statically."
         ; Pp.text "This is not allowed by dune"
-        ] )
+        ])
   | Setenv (var, value, t) ->
     let var =
       match E.string ~expander var with
@@ -300,9 +300,9 @@ let rec partial_expand t ~expander : Partial.t =
   | Remove_tree x -> Remove_tree (E.target ~expander x)
   | Mkdir x ->
     let res = E.path ~expander x in
-    ( match res with
+    (match res with
     | Expanded path -> check_mkdir (String_with_vars.loc x) path
-    | Unexpanded _ -> () );
+    | Unexpanded _ -> ());
     Mkdir res
   | Digest_files x -> Digest_files (List.map x ~f:(E.path ~expander))
   | Diff { optional; file1; file2; mode } ->
@@ -319,14 +319,15 @@ let rec partial_expand t ~expander : Partial.t =
       , E.target ~expander target )
   | No_infer t -> No_infer (partial_expand t ~expander)
   | Pipe (outputs, l) -> Pipe (outputs, List.map l ~f:(partial_expand ~expander))
-  | Format_dune_file (src, dst) ->
-    Format_dune_file (E.path ~expander src, E.target ~expander dst)
+  | Format_dune_file (ver, src, dst) ->
+    Format_dune_file (ver, E.path ~expander src, E.target ~expander dst)
   | Cram script -> Cram (E.path ~expander script)
 
 module Infer : sig
   module Outcome : sig
     type t =
       { deps : Path.Set.t
+      ; deps_if_exist : Path.Set.t
       ; targets : Path.Build.Set.t
       }
   end
@@ -340,6 +341,7 @@ end = struct
   module Outcome = struct
     type t =
       { deps : Path.Set.t
+      ; deps_if_exist : Path.Set.t
       ; targets : Path.Build.Set.t
       }
   end
@@ -369,6 +371,7 @@ end = struct
 
     type t =
       { deps : path_set
+      ; deps_if_exist : path_set
       ; targets : target_set
       }
   end
@@ -386,11 +389,16 @@ end = struct
 
     val ( +< ) : outcome -> path -> outcome
 
+    (** Add a dependency *)
     val ( +<+ ) : outcome -> target -> outcome
 
     val ( +<! ) : outcome -> program -> outcome
 
+    (** Remove a target *)
     val ( +<- ) : outcome -> target -> outcome
+
+    (** Add an optional (if exists) dependency *)
+    val ( +<? ) : outcome -> path -> outcome
   end
 
   module Make
@@ -434,9 +442,9 @@ end = struct
       | Cram script -> acc +< script
       | Diff { optional; file1; file2; mode = _ } ->
         if optional then
-          acc +< file1 +<- file2
+          acc +<? file1 +<- file2
         else
-          acc +< file1 +<+ file2
+          acc +<? file1 +<+ file2
       | Merge_files_into (sources, _extras, target) ->
         List.fold_left sources ~init:acc ~f:( +< ) +@+ target
       | Echo _
@@ -446,17 +454,25 @@ end = struct
       | Mkdir _
       | No_infer _ ->
         acc
-      | Format_dune_file (src, dst) -> acc +< src +@+ dst
+      | Format_dune_file (_, src, dst) -> acc +< src +@+ dst
 
     let infer t =
-      let { deps; targets } =
-        infer { deps = Sets.Deps.empty; targets = Sets.Targets.empty } t
+      let { deps; deps_if_exist; targets } =
+        infer
+          { deps = Sets.Deps.empty
+          ; deps_if_exist = Sets.Deps.empty
+          ; targets = Sets.Targets.empty
+          }
+          t
       in
 
       (* A file can be inferred as both a dependency and a target, for instance:
 
          {[ (progn (copy a b) (copy b c)) ]} *)
-      { deps = Sets.Deps.diff deps targets; targets }
+      { deps = Sets.Deps.diff deps targets
+      ; deps_if_exist = Sets.Deps.diff deps_if_exist targets
+      ; targets
+      }
   end
   [@@inline always]
 
@@ -472,24 +488,28 @@ end = struct
     end
   end
 
-  include Make (Action.Ast) (Sets) (Outcome)
-            (struct
-              let ( +@+ ) acc fn =
-                { acc with targets = Path.Build.Set.add acc.targets fn }
+  include
+    Make (Action.Ast) (Sets) (Outcome)
+      (struct
+        let ( +@+ ) acc fn =
+          { acc with targets = Path.Build.Set.add acc.targets fn }
 
-              let ( +< ) acc fn = { acc with deps = Path.Set.add acc.deps fn }
+        let ( +< ) acc fn = { acc with deps = Path.Set.add acc.deps fn }
 
-              let ( +<+ ) acc fn =
-                { acc with deps = Path.Set.add acc.deps (Path.build fn) }
+        let ( +<+ ) acc fn =
+          { acc with deps = Path.Set.add acc.deps (Path.build fn) }
 
-              let ( +<- ) acc fn =
-                { acc with targets = Path.Build.Set.remove acc.targets fn }
+        let ( +<? ) acc fn =
+          { acc with deps_if_exist = Path.Set.add acc.deps_if_exist fn }
 
-              let ( +<! ) acc prog =
-                match prog with
-                | Ok p -> acc +< p
-                | Error _ -> acc
-            end)
+        let ( +<- ) acc fn =
+          { acc with targets = Path.Build.Set.remove acc.targets fn }
+
+        let ( +<! ) acc prog =
+          match prog with
+          | Ok p -> acc +< p
+          | Error _ -> acc
+      end)
 
   module Partial_with_all_targets =
     Make (Partial.Past) (Sets) (Outcome)
@@ -513,6 +533,12 @@ end = struct
             { acc with deps = Path.Set.add acc.deps (Path.build fn) }
           | Unexpanded _ -> acc
 
+        let ( +<? ) acc (fn : _ String_with_vars.Partial.t) =
+          match fn with
+          | Expanded fn ->
+            { acc with deps_if_exist = Path.Set.add acc.deps_if_exist fn }
+          | Unexpanded _ -> acc
+
         let ( +<- ) acc (fn : _ String_with_vars.Partial.t) =
           match fn with
           | Expanded fn ->
@@ -547,6 +573,12 @@ end = struct
             { acc with deps = Path.Set.add acc.deps (Path.build fn) }
           | Unexpanded _ -> acc
 
+        let ( +<? ) acc (fn : _ String_with_vars.Partial.t) =
+          match fn with
+          | Expanded fn ->
+            { acc with deps_if_exist = Path.Set.add acc.deps_if_exist fn }
+          | Unexpanded _ -> acc
+
         let ( +<- ) acc (fn : _ String_with_vars.Partial.t) =
           match fn with
           | Expanded fn ->
@@ -595,6 +627,7 @@ end = struct
   module Outcome_unexp = struct
     type t =
       { deps : S_unexp.Deps.t
+      ; deps_if_exist : S_unexp.Deps.t
       ; targets : S_unexp.Targets.t
       }
   end
@@ -614,6 +647,8 @@ end = struct
 
         let ( +<+ ) acc _ = acc
 
+        let ( +<? ) acc _ = acc
+
         let ( +<! ) = ( +< )
 
         let ( +<- ) acc fn =
@@ -626,10 +661,21 @@ end = struct
   let unexpanded_targets t = (Unexp.infer t).targets
 end
 
+let add_deps_if_exist deps_if_exist =
+  let open Build.O in
+  (let+ l =
+     Path.Set.to_list_map deps_if_exist ~f:(fun f ->
+         Build.if_file_exists f ~then_:(Build.return (Some f))
+           ~else_:(Build.return None))
+     |> Build.all
+   in
+   List.filter_opt l)
+  |> Build.dyn_paths_unit
+
 let expand t ~loc ~dep_kind ~targets_dir ~targets:targets_written_by_user
     ~expander deps_written_by_user =
   let open Build.O in
-  ( match (targets_written_by_user : Targets.Or_forbidden.t) with
+  (match (targets_written_by_user : Targets.Or_forbidden.t) with
   | Targets _ -> ()
   | Forbidden context -> (
     match Infer.unexpanded_targets t with
@@ -637,15 +683,14 @@ let expand t ~loc ~dep_kind ~targets_dir ~targets:targets_written_by_user
     | x :: _ ->
       let loc = String_with_vars.loc x in
       User_error.raise ~loc
-        [ Pp.textf "%s must not have targets." (String.capitalize context) ] )
-  );
+        [ Pp.textf "%s must not have targets." (String.capitalize context) ]));
   let partially_expanded, fully_expanded =
     Expander.expand_action expander ~dep_kind ~deps_written_by_user
       ~targets_written_by_user
       ~partial:(fun expander -> partial_expand t ~expander)
       ~final:(fun expander t -> Partial.expand t ~expander)
   in
-  let { Infer.Outcome.deps; targets } =
+  let { Infer.Outcome.deps; deps_if_exist; targets } =
     Infer.partial targets_written_by_user partially_expanded
   in
   Path.Build.Set.iter targets ~f:(fun target ->
@@ -657,25 +702,35 @@ let expand t ~loc ~dep_kind ~targets_dir ~targets:targets_written_by_user
           ; Pp.enumerate (Path.Build.Set.to_list targets) ~f:(fun target ->
                 Pp.text (Dpath.describe_path (Path.build target)))
           ]);
-  let targets = Path.Build.Set.to_list targets in
   Build.path_set deps
+  >>> add_deps_if_exist deps_if_exist
   >>> Build.dyn_path_set
-        (let+ action =
-           let+ unresolved = fully_expanded in
-           let artifacts = Expander.artifacts expander in
-           Action.Unresolved.resolve unresolved ~f:(fun loc prog ->
-               Artifacts.Bin.binary artifacts ~loc prog |> Action.Prog.ok_exn)
+        (let+ (action, deps), deps_if_exist_which_exist =
+           Build.filter_existing_files
+             (let+ action =
+                let+ unresolved = fully_expanded in
+                let artifacts = Expander.artifacts expander in
+                Action.Unresolved.resolve unresolved ~f:(fun loc prog ->
+                    Artifacts.Bin.binary artifacts ~loc prog
+                    |> Action.Prog.ok_exn)
+              in
+
+              (* Targets cannot be introduced at this stage because they must
+                 all be expanded after partial expansion.
+
+                 This is because we don't allow things like: [(with-stdout-to
+                 %{read:foo} ..)] *)
+              let { Infer.Outcome.deps
+                  ; Infer.Outcome.deps_if_exist
+                  ; targets = _
+                  } =
+                Infer.infer action
+              in
+              let dir = Path.build (Expander.dir expander) in
+              ((Action.Chdir (dir, action), deps), deps_if_exist))
          in
-
-         (* Targets cannot be introduced at this stage because they must all be
-            expanded after partial expansion.
-
-            This is because we don't allow things like: [(with-stdout-to
-            %{read:foo} ..)] *)
-         let { Infer.Outcome.deps; targets = _ } = Infer.infer action in
-         let dir = Path.build (Expander.dir expander) in
-         (Action.Chdir (dir, action), deps))
-  |> Build.with_targets ~targets
+         (action, Path.Set.union deps deps_if_exist_which_exist))
+  |> Build.with_targets_set ~targets
 
 (* We re-export [Action_dune_lang] in the end to avoid polluting the inferred
    types in this module with all the various t's *)
