@@ -2,6 +2,10 @@ open! Dune_engine
 open Import
 open! No_io
 
+let def name dyn =
+  let open Pp.O in
+  Pp.box ~indent:2 (Pp.textf "let %s = " name ++ Dyn.pp dyn)
+
 let rule sctx compile (exes : Dune_file.Executables.t) () =
   let libs = Result.ok_exn (Lazy.force (Lib.Compile.requires_link compile)) in
   let locals, externals =
@@ -10,11 +14,7 @@ let rule sctx compile (exes : Dune_file.Executables.t) () =
         | Some x -> Left x
         | None -> Right lib)
   in
-  let open Pp.O in
-  let def name dyn =
-    Pp.box ~indent:2 (Pp.textf "let %s = " name ++ Dyn.pp dyn)
-  in
-  Format.asprintf "%a@." Pp.render_ignore_tags
+  Format.asprintf "%a@." Pp.to_fmt
     (Pp.vbox
        (Pp.concat ~sep:Pp.cut
           [ def "executables"
@@ -32,7 +32,7 @@ let rule sctx compile (exes : Dune_file.Executables.t) () =
                  (List.map locals ~f:(fun x ->
                       let info = Lib.Local.info x in
                       let dir = Lib_info.src_dir info in
-                      let special_builton_support =
+                      let special_builtin_support =
                         match Lib_info.special_builtin_support info with
                         | Some (Build_info { data_module; _ }) ->
                           Some data_module
@@ -42,15 +42,16 @@ let rule sctx compile (exes : Dune_file.Executables.t) () =
                         [ Path.Source.to_dyn
                             (Path.Build.drop_build_context_exn dir)
                         ; Dyn.Encoder.option Module_name.to_dyn
-                            ( match Lib_info.main_module_name info with
+                            (match Lib_info.main_module_name info with
                             | From _ -> None
-                            | This x -> x )
-                        ; ( match
-                              Dir_contents.get sctx ~dir |> Dir_contents.dirs
-                            with
+                            | This x -> x)
+                        ; (match
+                             Dir_contents.get sctx ~dir |> Dir_contents.dirs
+                           with
                           | _ :: _ :: _ -> Dyn.Bool true
-                          | _ -> Dyn.Bool false )
-                        ; Dyn.Encoder.(option string) special_builton_support
+                          | _ -> Dyn.Bool false)
+                        ; Dyn.Encoder.option Module_name.to_dyn
+                            special_builtin_support
                         ])))
           ]))
 
