@@ -87,23 +87,24 @@ module T = struct
     ; env : Env.t
     ; findlib : Findlib.t
     ; findlib_toolchain : Context_name.t option
+    ; default_ocamlpath : Path.t list
     ; arch_sixtyfour : bool
     ; install_prefix : Path.t Memo.Lazy.Async.t
     ; ocaml_config : Ocaml_config.t
+    ; ocaml_config_vars : Ocaml_config.Vars.t
     ; version : Ocaml_version.t
     ; stdlib_dir : Path.t
     ; supports_shared_libraries : Dynlink_supported.By_the_os.t
     ; which : string -> Path.t option
     ; lib_config : Lib_config.t
+    ; build_context : Build_context.t
     }
 
   let equal x y = Context_name.equal x.name y.name
 
   let hash t = Context_name.hash t.name
 
-  let rec to_build_context { name; build_dir; env; for_host; _ } =
-    Build_context.create ~name ~build_dir ~env
-      ~host:(Option.map ~f:to_build_context for_host)
+  let build_context t = t.build_context
 
   let to_dyn t : Dyn.t =
     let open Dyn.Encoder in
@@ -155,7 +156,7 @@ let read_opam_config_var ~env (var : string) : string option Fiber.t =
       [ "config"; "var"; var ]
     >>| function
     | Ok s -> Some (String.trim s)
-    | Error _ -> None )
+    | Error _ -> None)
 
 let ocamlpath_sep =
   if Sys.cygwin then
@@ -183,14 +184,14 @@ module Build_environment_kind = struct
       | Opam _ -> (
         match opam_prefix with
         | Some s -> Opam2_environment s
-        | None -> Opam1_environment )
+        | None -> Opam1_environment)
       | Default -> (
         match Setup.library_path with
         | Some l -> Hardcoded_path l
         | None -> (
           match opam_prefix with
           | Some s -> Opam2_environment s
-          | None -> Unknown ) ) )
+          | None -> Unknown)))
 end
 
 let ocamlfind_printconf_path ~env ~ocamlfind ~toolchain =
@@ -272,6 +273,9 @@ let write_dot_dune_dir ~build_dir ~ocamlc ~ocaml_config_vars =
   in
   ()
 
+let init_configurator { build_dir; ocamlc; ocaml_config_vars; _ } =
+  write_dot_dune_dir ~build_dir ~ocamlc ~ocaml_config_vars
+
 let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
     ~host_context ~host_toolchain ~profile ~fdo_target_exe
     ~dynamically_linked_foreign_archives ~instrument_with =
@@ -298,10 +302,10 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
            contents of the variable, but "ocamlfind printconf conf" still prints
            the configuration file set at the configuration time of ocamlfind,
            sigh... *)
-        ( match Env.get env "OCAMLFIND_CONF" with
+        (match Env.get env "OCAMLFIND_CONF" with
         | Some s -> Fiber.return s
         | None ->
-          Process.run_capture_line ~env Strict fn [ "printconf"; "conf" ] )
+          Process.run_capture_line ~env Strict fn [ "printconf"; "conf" ])
         >>| Path.of_filename_relative_to_initial_cwd)
   in
   let create_one ~(name : Context_name.t) ~implicit ~findlib_toolchain ~host
@@ -336,7 +340,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       | None -> (
         match which "ocamlc" with
         | Some x -> x
-        | None -> prog_not_found_in_path "ocamlc" )
+        | None -> prog_not_found_in_path "ocamlc")
     in
     let dir = Path.parent_exn ocamlc in
     let get_ocaml_tool prog =
@@ -353,7 +357,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
           in
           Error
             (Action.Prog.Not_found.create ~context:name ~program:prog ~loc:None
-               ~hint ()) )
+               ~hint ()))
     in
     let build_dir = Context_name.build_dir name in
     let ocamlpath =
@@ -370,7 +374,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
           | None, None -> None
           | Some s, None -> Some s
           | None, Some _ -> None
-          | Some x, Some y -> Option.some_if (x <> y) x )
+          | Some x, Some y -> Option.some_if (x <> y) x)
       with
       | None -> []
       | Some s -> Bin.parse_path s ~sep:ocamlpath_sep
@@ -379,29 +383,28 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       Memo.lazy_async ~cutoff:(Option.equal String.equal) (fun () ->
           read_opam_config_var ~env "lib")
     in
-    let findlib_paths () =
+    let default_findlib_paths () =
       match Build_environment_kind.query ~kind ~findlib_toolchain ~env with
       | Cross_compilation_using_findlib_toolchain toolchain ->
         let ocamlfind = which_exn "ocamlfind" in
+        let env = Env.remove env ~var:"OCAMLPATH" in
         ocamlfind_printconf_path ~env ~ocamlfind ~toolchain:(Some toolchain)
       | Hardcoded_path l ->
-        Fiber.return
-          (ocamlpath @ List.map l ~f:Path.of_filename_relative_to_initial_cwd)
+        Fiber.return (List.map l ~f:Path.of_filename_relative_to_initial_cwd)
       | Opam2_environment opam_prefix ->
         let p = Path.of_filename_relative_to_initial_cwd opam_prefix in
         let p = Path.relative p "lib" in
-        Fiber.return (ocamlpath @ [ p ])
+        Fiber.return [ p ]
       | Opam1_environment -> (
         Memo.Lazy.Async.force opam_config_var_lib >>| function
-        | Some s -> ocamlpath @ [ Path.of_filename_relative_to_initial_cwd s ]
-        | None -> Utils.program_not_found "opam" ~loc:None )
+        | Some s -> [ Path.of_filename_relative_to_initial_cwd s ]
+        | None -> Utils.program_not_found "opam" ~loc:None)
       | Unknown -> (
         match which "ocamlfind" with
         | Some ocamlfind ->
+          let env = Env.remove env ~var:"OCAMLPATH" in
           ocamlfind_printconf_path ~env ~ocamlfind ~toolchain:None
-        | None ->
-          Fiber.return
-            (ocamlpath @ [ Path.relative (Path.parent_exn dir) "lib" ]) )
+        | None -> Fiber.return [ Path.relative (Path.parent_exn dir) "lib" ])
     in
     let ocaml_config_ok_exn = function
       | Ok x -> x
@@ -414,18 +417,20 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       | Error (Makefile_config file, msg) ->
         User_error.raise ~loc:(Loc.in_file file) [ Pp.text msg ]
     in
-    let* findlib_paths, ocfg =
-      Fiber.fork_and_join findlib_paths (fun () ->
+    let* default_ocamlpath, (ocaml_config_vars, ocfg) =
+      Fiber.fork_and_join default_findlib_paths (fun () ->
           let+ lines =
             Process.run_capture_lines ~env Strict ocamlc [ "-config" ]
           in
           ocaml_config_ok_exn
-            ( match Ocaml_config.Vars.of_lines lines with
+            (match Ocaml_config.Vars.of_lines lines with
             | Ok vars ->
-              write_dot_dune_dir ~build_dir ~ocamlc ~ocaml_config_vars:vars;
-              Ocaml_config.make vars
-            | Error msg -> Error (Ocamlc_config, msg) ))
+              let open Result.O in
+              let+ ocfg = Ocaml_config.make vars in
+              (vars, ocfg)
+            | Error msg -> Error (Ocamlc_config, msg)))
     in
+    let findlib_paths = ocamlpath @ default_ocamlpath in
     let version = Ocaml_version.of_ocaml_config ocfg in
     let env =
       (* See comment in ansi_color.ml for setup_env_for_colors. For versions
@@ -450,29 +455,31 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
     let env =
       let cwd = Sys.getcwd () in
       let extend_var var ?(path_sep = Bin.path_sep) v =
-        let v = Filename.concat cwd (Path.to_string v) in
+        let v = Filename.concat cwd (Path.Build.to_string v) in
         match Env.get env var with
         | None -> (var, v)
         | Some prev -> (var, sprintf "%s%c%s" v path_sep prev)
       in
       let vars =
-        let local_lib_path =
-          Path.Build.relative (Config.local_install_dir ~context:name) "lib"
-          |> Path.build
-        in
+        let local_lib_root = Config.local_install_lib_root ~context:name in
         [ extend_var "CAML_LD_LIBRARY_PATH"
-            (Path.build
-               (Path.Build.relative
-                  (Config.local_install_dir ~context:name)
-                  "lib/stublibs"))
-        ; extend_var "OCAMLPATH" ~path_sep:ocamlpath_sep local_lib_path
+            (Path.Build.relative
+               (Config.local_install_dir ~context:name)
+               "lib/stublibs")
+        ; extend_var "OCAMLPATH" ~path_sep:ocamlpath_sep local_lib_root
+        ; ("DUNE_OCAML_STDLIB", Ocaml_config.standard_library ocfg)
+        ; ( "DUNE_OCAML_HARDCODED"
+          , String.concat
+              ~sep:(Char.escaped ocamlpath_sep)
+              (List.map ~f:Path.to_string default_ocamlpath) )
         ; extend_var "OCAMLTOP_INCLUDE_PATH"
-            (Path.relative local_lib_path "toplevel")
+            (Path.Build.relative local_lib_root "toplevel")
         ; extend_var "OCAMLFIND_IGNORE_DUPS_IN" ~path_sep:ocamlpath_sep
-            local_lib_path
-        ; extend_var "MANPATH"
-            (Path.build (Config.local_install_man_dir ~context:name))
+            local_lib_root
+        ; extend_var "MANPATH" (Config.local_install_man_dir ~context:name)
         ; ("INSIDE_DUNE", Path.to_absolute_filename (Path.build build_dir))
+        ; ( "DUNE_SOURCEROOT"
+          , Path.to_absolute_filename (Path.source Path.Source.root) )
         ]
       in
       Env.extend env ~vars:(Env.Map.of_list_exn vars)
@@ -480,8 +487,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
              match host with
              | None ->
                let _key, path =
-                 Path.build (Config.local_install_bin_dir ~context:name)
-                 |> extend_var "PATH"
+                 Config.local_install_bin_dir ~context:name |> extend_var "PATH"
                in
                Some path
              | Some host -> Env.get host.env "PATH")
@@ -511,6 +517,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       ; ocaml_version_string = Ocaml_config.version_string ocfg
       ; ocaml_version = Ocaml_version.of_ocaml_config ocfg
       ; instrument_with
+      ; context_name = name
       }
     in
     if Option.is_some fdo_target_exe then
@@ -536,6 +543,11 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       supports_shared_libraries && dynamically_linked_foreign_archives
     in
     let t =
+      let build_context =
+        Build_context.create ~name ~build_dir ~env ~stdlib_dir
+          ~default_ocamlpath
+          ~host:(Option.map host ~f:(fun c -> c.build_context))
+      in
       { name
       ; implicit
       ; kind
@@ -561,15 +573,18 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       ; env
       ; findlib = Findlib.create ~paths:findlib_paths ~lib_config
       ; findlib_toolchain
+      ; default_ocamlpath
       ; arch_sixtyfour
       ; install_prefix
       ; stdlib_dir
       ; ocaml_config = ocfg
+      ; ocaml_config_vars
       ; version
       ; supports_shared_libraries =
           Dynlink_supported.By_the_os.of_bool supports_shared_libraries
       ; which
       ; lib_config
+      ; build_context
       }
     in
     if Ocaml_version.supports_response_file version then (
@@ -666,14 +681,14 @@ let create_for_opam ~root ~env ~env_nodes ~targets ~profile ~switch ~name
   let args =
     List.concat
       [ [ "config"; "env" ]
-      ; ( match root with
+      ; (match root with
         | None -> []
-        | Some root -> [ "--root"; root ] )
+        | Some root -> [ "--root"; root ])
       ; [ "--switch"; switch; "--sexp" ]
-      ; ( if version < (2, 0, 0) then
+      ; (if version < (2, 0, 0) then
           []
         else
-          [ "--set-switch" ] )
+          [ "--set-switch" ])
       ]
   in
   let* s = Process.run_capture ~env Strict opam args in
@@ -728,6 +743,7 @@ let instantiate_context env (workspace : Workspace.t)
       ; fdo_target_exe
       ; dynamically_linked_foreign_archives
       ; instrument_with
+      ; merlin = _
       } ->
     let merlin =
       workspace.merlin_context = Some (Workspace.Context.name context)
@@ -757,10 +773,10 @@ let instantiate_context env (workspace : Workspace.t)
           ; fdo_target_exe
           ; dynamically_linked_foreign_archives
           ; instrument_with
+          ; merlin
           }
       ; switch
       ; root
-      ; merlin
       } ->
     let env = extend_paths ~env paths in
     create_for_opam ~root ~env_nodes ~env ~profile ~switch ~name ~merlin
@@ -812,7 +828,7 @@ module Create = struct
                 | [ x ] -> Some x
                 | [] -> assert false (* checked by workspace *)
                 | _ :: _ -> assert false
-                (* target cannot be host *) )
+                (* target cannot be host *))
             in
             instantiate_context env workspace ~context ~host_context
           in
@@ -820,11 +836,11 @@ module Create = struct
           let ivar = Context_name.Map.find_exn context_ivars_map name in
           Fiber.Ivar.fill ivar contexts)
     in
-    let* contexts =
-      Fiber.parallel_map (Context_name.Map.values context_ivars_map)
-        ~f:(fun ivar -> Fiber.Ivar.read ivar)
+    let+ contexts =
+      Context_name.Map.values context_ivars_map
+      |> Fiber.parallel_map ~f:Fiber.Ivar.read
     in
-    Fiber.return (List.concat contexts)
+    List.concat contexts
 
   let memo =
     Memo.create "create-context" ~doc:"create contexts"
@@ -875,7 +891,7 @@ let install_ocaml_libdir t =
         Process.run_capture_line ~env:t.env Strict fn [ "printconf"; "destdir" ]
       in
       Some (Path.of_filename_relative_to_initial_cwd s)
-    | None -> Fiber.return None )
+    | None -> Fiber.return None)
 
 let compiler t (mode : Mode.t) =
   match mode with
@@ -906,4 +922,4 @@ let map_exe (context : t) =
       match Path.extract_build_context_dir exe with
       | Some (dir, exe) when Path.equal dir (Path.build context.build_dir) ->
         Path.append_source (Path.build host.build_dir) exe
-      | _ -> exe )
+      | _ -> exe)
