@@ -14,5 +14,7 @@ val data : 'data t -> 'data
 
 val map : 'a t -> f:('a -> 'b) -> 'b t
 
+(** [deep_fold dwd ~init ~f] fold the most inner elements but still gives the
+    corresponding directory information corresponding as first argument *)
 val deep_fold :
   'a list t list -> init:'acc -> f:('a list t -> 'a -> 'acc -> 'acc) -> 'acc
