@@ -2,7 +2,6 @@ open! Dune_engine
 open! Stdune
 open Import
 open Dune_lang.Decoder
-open Stanza_common
 
 (* This file defines Dune types as well as the S-expression syntax for the
    various supported versions of the specification. *)
@@ -46,6 +45,10 @@ module Js_of_ocaml = struct
     { flags = Ordered_set_lang.Unexpanded.standard; javascript_files = [] }
 end
 
+type for_ =
+  | Executable
+  | Library of Wrapped.t option
+
 module Lib_deps = struct
   type t = Lib_dep.t list
 
@@ -54,9 +57,16 @@ module Lib_deps = struct
     | Optional
     | Forbidden
 
-  let decode ~allow_re_export =
+  let decode for_ =
     let+ loc = loc
-    and+ t = repeat (Lib_dep.decode ~allow_re_export) in
+    and+ t =
+      let allow_re_export =
+        match for_ with
+        | Library _ -> true
+        | Executable -> false
+      in
+      repeat (Lib_dep.decode ~allow_re_export)
+    in
     let add kind name acc =
       match Lib_name.Map.find acc name with
       | None -> Lib_name.Map.set acc name kind
@@ -81,22 +91,22 @@ module Lib_deps = struct
                 "library %S is present both as a forbidden and required \
                  dependency"
                 (Lib_name.to_string name)
-            ] )
+            ])
     in
     ignore
-      ( List.fold_left t ~init:Lib_name.Map.empty ~f:(fun acc x ->
-            match x with
-            | Lib_dep.Re_export (_, s)
-            | Lib_dep.Direct (_, s) ->
-              add Required s acc
-            | Select { choices; _ } ->
-              List.fold_left choices ~init:acc
-                ~f:(fun acc (c : Lib_dep.Select.Choice.t) ->
-                  let acc =
-                    Lib_name.Set.fold c.required ~init:acc ~f:(add Optional)
-                  in
-                  Lib_name.Set.fold c.forbidden ~init:acc ~f:(add Forbidden)))
-        : kind Lib_name.Map.t );
+      (List.fold_left t ~init:Lib_name.Map.empty ~f:(fun acc x ->
+           match x with
+           | Lib_dep.Re_export (_, s)
+           | Lib_dep.Direct (_, s) ->
+             add Required s acc
+           | Select { choices; _ } ->
+             List.fold_left choices ~init:acc
+               ~f:(fun acc (c : Lib_dep.Select.Choice.t) ->
+                 let acc =
+                   Lib_name.Set.fold c.required ~init:acc ~f:(add Optional)
+                 in
+                 Lib_name.Set.fold c.forbidden ~init:acc ~f:(add Forbidden)))
+        : kind Lib_name.Map.t);
     t
 
   let of_pps pps = List.map pps ~f:(fun pp -> Lib_dep.direct (Loc.none, pp))
@@ -104,7 +114,7 @@ module Lib_deps = struct
   let info t ~kind =
     List.concat_map t ~f:(function
       | Lib_dep.Re_export (_, s)
-      | Lib_dep.Direct (_, s) ->
+      | Direct (_, s) ->
         [ (s, kind) ]
       | Select { choices; _ } ->
         List.concat_map choices ~f:(fun (c : Lib_dep.Select.Choice.t) ->
@@ -162,18 +172,18 @@ module Buildable = struct
     ; flags : Ocaml_flags.Spec.t
     ; js_of_ocaml : Js_of_ocaml.t
     ; allow_overlapping_dependencies : bool
+    ; root_module : (Loc.t * Module_name.t) option
     }
 
-  let decode ~in_library ~allow_re_export =
+  let decode (for_ : for_) =
     let use_foreign =
       Dune_lang.Syntax.deleted_in Stanza.syntax (2, 0)
         ~extra_info:"Use the (foreign_stubs ...) field instead."
     in
     let only_in_library decode =
-      if in_library then
-        decode
-      else
-        return None
+      match for_ with
+      | Executable -> return None
+      | Library _ -> decode
     in
     let add_stubs language ~loc ~names ~flags foreign_stubs =
       match names with
@@ -193,8 +203,8 @@ module Buildable = struct
         (Dune_lang.Syntax.since Stanza.syntax (2, 0) >>> Foreign.Stubs.decode)
     and+ foreign_archives =
       field_o "foreign_archives"
-        ( Dune_lang.Syntax.since Stanza.syntax (2, 0)
-        >>> repeat (located Foreign.Archive.decode) )
+        (Dune_lang.Syntax.since Stanza.syntax (2, 0)
+        >>> repeat (located Foreign.Archive.decode))
     and+ c_flags =
       only_in_library
         (field_o "c_flags" (use_foreign >>> Ordered_set_lang.Unexpanded.decode))
@@ -210,18 +220,17 @@ module Buildable = struct
       located
         (only_in_library
            (field_o "cxx_names" (use_foreign >>> Ordered_set_lang.decode)))
-    and+ modules = modules_field "modules"
+    and+ modules = Stanza_common.modules_field "modules"
     and+ self_build_stubs_archive_loc, self_build_stubs_archive =
       located
         (only_in_library
            (field ~default:None "self_build_stubs_archive"
-              ( Dune_lang.Syntax.deleted_in Stanza.syntax (2, 0)
-                  ~extra_info:"Use the (foreign_archives ...) field instead."
-              >>> enter (maybe string) )))
+              (Dune_lang.Syntax.deleted_in Stanza.syntax (2, 0)
+                 ~extra_info:"Use the (foreign_archives ...) field instead."
+              >>> enter (maybe string))))
     and+ modules_without_implementation =
-      modules_field "modules_without_implementation"
-    and+ libraries =
-      field "libraries" (Lib_deps.decode ~allow_re_export) ~default:[]
+      Stanza_common.modules_field "modules_without_implementation"
+    and+ libraries = field "libraries" (Lib_deps.decode for_) ~default:[]
     and+ flags = Ocaml_flags.Spec.decode
     and+ js_of_ocaml =
       field "js_of_ocaml" Js_of_ocaml.decode ~default:Js_of_ocaml.default
@@ -231,8 +240,40 @@ module Buildable = struct
     and+ loc_instrumentation, instrumentation =
       located
         (multi_field "instrumentation"
-           ( Dune_lang.Syntax.since Stanza.syntax (2, 7)
-           >>> fields (field "backend" (located Lib_name.decode)) ))
+           (Dune_lang.Syntax.since Stanza.syntax (2, 7)
+           >>> fields
+                 (let+ backend =
+                    field "backend"
+                      (let+ libname = located Lib_name.decode
+                       and+ flags =
+                         let* current_ver =
+                           Dune_lang.Syntax.get_exn Stanza.syntax
+                         in
+                         let version_check flag =
+                           let ver = (2, 8) in
+                           if current_ver >= ver then
+                             flag
+                           else
+                             let what =
+                               "The possibility to pass arguments to \
+                                instrumentation backends"
+                             in
+                             Dune_lang.Syntax.Error.since
+                               (String_with_vars.loc flag)
+                               Stanza.syntax ver ~what
+                         in
+                         repeat (String_with_vars.decode >>| version_check)
+                       in
+                       (libname, flags))
+                  and+ deps =
+                    field "deps" ~default:[]
+                      (Dune_lang.Syntax.since Stanza.syntax (2, 9)
+                      >>> repeat Dep_conf.decode)
+                  in
+                  (backend, deps))))
+    and+ root_module =
+      field_o "root_module"
+        (Dune_lang.Syntax.since Stanza.syntax (2, 8) >>> Module_name.decode_loc)
     in
     let preprocess =
       let init =
@@ -240,7 +281,9 @@ module Buildable = struct
         Module_name.Per_item.map preprocess ~f:(Preprocess.map ~f)
       in
       List.fold_left instrumentation
-        ~f:(Preprocess.Per_module.add_instrumentation ~loc:loc_instrumentation)
+        ~f:(fun accu ((backend, flags), deps) ->
+          Preprocess.Per_module.add_instrumentation accu
+            ~loc:loc_instrumentation ~flags ~deps backend)
         ~init
     in
     let foreign_stubs =
@@ -286,6 +329,7 @@ module Buildable = struct
     ; flags
     ; js_of_ocaml
     ; allow_overlapping_dependencies
+    ; root_module
     }
 
   let has_foreign t =
@@ -327,14 +371,14 @@ module Public_lib = struct
     match x with
     | Some x -> Ok x
     | None ->
-      Pkg.resolve project pkg
+      Stanza_common.Pkg.resolve project pkg
       |> Result.map ~f:(fun pkg ->
              { package = pkg
              ; sub_dir =
-                 ( if rest = [] then
+                 (if rest = [] then
                    None
                  else
-                   Some (String.concat rest ~sep:"/") )
+                   Some (String.concat rest ~sep:"/"))
              ; name = loc_name
              })
 
@@ -484,14 +528,14 @@ module Library = struct
 
     let make ~wrapped ~implements ~special_builtin_support :
         t Lib_info.Inherited.t =
-      ( match (wrapped, special_builtin_support) with
+      (match (wrapped, special_builtin_support) with
       | Some (loc, Yes_with_transition _), Some _ ->
         User_error.raise ~loc
           [ Pp.text
               "Cannot have transition modules for libraries with special \
                builtin support"
           ]
-      | _, _ -> () );
+      | _, _ -> ());
       match (wrapped, implements) with
       | None, None -> This default
       | None, Some w -> From w
@@ -506,9 +550,13 @@ module Library = struct
     let field = field_o "wrapped" (located decode)
   end
 
+  type visibility =
+    | Public of Public_lib.t
+    | Private of Package.t option
+
   type t =
     { name : Loc.t * Lib_name.Local.t
-    ; public : Public_lib.t option
+    ; visibility : visibility
     ; synopsis : string option
     ; install_c_headers : string list
     ; ppx_runtime_libraries : (Loc.t * Lib_name.t) list
@@ -537,8 +585,9 @@ module Library = struct
   let decode =
     fields
       (let* stanza_loc = loc in
+       let* wrapped = Wrapped.field in
        let* dune_version = Dune_lang.Syntax.get_exn Stanza.syntax in
-       let+ buildable = Buildable.decode ~in_library:true ~allow_re_export:true
+       let+ buildable = Buildable.decode (Library (Option.map ~f:snd wrapped))
        and+ name = field_o "name" Lib_name.Local.decode_loc
        and+ public =
          field_o "public_name" (Public_lib.decode ~allow_deprecated_names:false)
@@ -558,7 +607,6 @@ module Library = struct
          field "modes" Mode_conf.Set.decode
            ~default:(Mode_conf.Set.default stanza_loc)
        and+ kind = field "kind" Lib_kind.decode ~default:Lib_kind.Normal
-       and+ wrapped = Wrapped.field
        and+ optional = field_b "optional"
        and+ no_dynlink = field_b "no_dynlink"
        and+ () =
@@ -576,36 +624,40 @@ module Library = struct
        and+ project = Dune_project.get_exn ()
        and+ virtual_modules =
          field_o "virtual_modules"
-           ( Dune_lang.Syntax.since Stanza.syntax (1, 7)
-           >>> Ordered_set_lang.decode )
+           (Dune_lang.Syntax.since Stanza.syntax (1, 7)
+           >>> Ordered_set_lang.decode)
        and+ implements =
          field_o "implements"
-           ( Dune_lang.Syntax.since Stanza.syntax (1, 7)
-           >>> located Lib_name.decode )
+           (Dune_lang.Syntax.since Stanza.syntax (1, 7)
+           >>> located Lib_name.decode)
        and+ default_implementation =
          field_o "default_implementation"
-           ( Dune_lang.Syntax.since Stanza.syntax (2, 6)
-           >>> located Lib_name.decode )
+           (Dune_lang.Syntax.since Stanza.syntax (2, 6)
+           >>> located Lib_name.decode)
        and+ private_modules =
          field_o "private_modules"
            (let* () = Dune_lang.Syntax.since Stanza.syntax (1, 2) in
             Ordered_set_lang.decode)
        and+ stdlib =
          field_o "stdlib"
-           ( Dune_lang.Syntax.since Ocaml_stdlib.syntax (0, 1)
-           >>> Ocaml_stdlib.decode )
+           (Dune_lang.Syntax.since Ocaml_stdlib.syntax (0, 1)
+           >>> Ocaml_stdlib.decode)
        and+ special_builtin_support =
          field_o "special_builtin_support"
-           ( Dune_lang.Syntax.since Stanza.syntax (1, 10)
-           >>> Lib_info.Special_builtin_support.decode )
+           (Dune_lang.Syntax.since Stanza.syntax (1, 10)
+           >>> Lib_info.Special_builtin_support.decode)
        and+ enabled_if =
          let open Enabled_if in
          let allowed_vars = Only Lib_config.allowed_in_enabled_if in
          decode ~allowed_vars ~since:(Some (1, 10)) ()
        and+ instrumentation_backend =
          field_o "instrumentation.backend"
-           ( Dune_lang.Syntax.since Stanza.syntax (2, 7)
-           >>> fields (field "ppx" (located Lib_name.decode)) )
+           (Dune_lang.Syntax.since Stanza.syntax (2, 7)
+           >>> fields (field "ppx" (located Lib_name.decode)))
+       and+ package =
+         field_o "package"
+           (Dune_lang.Syntax.since Stanza.syntax (2, 8)
+           >>> located Stanza_common.Pkg.decode)
        in
        let wrapped =
          Wrapped.make ~wrapped ~implements ~special_builtin_support
@@ -636,10 +688,23 @@ module Library = struct
          | None, None ->
            User_error.raise ~loc:stanza_loc
              [ Pp.text
-                 ( if dune_version >= (1, 1) then
+                 (if dune_version >= (1, 1) then
                    "supply at least least one of name or public_name fields"
                  else
-                   "name field is missing" )
+                   "name field is missing")
+             ]
+       in
+       let visibility =
+         match (public, package) with
+         | None, None -> Private None
+         | Some public, None -> Public public
+         | None, Some (_loc, package) -> Private (Some package)
+         | Some public, Some (loc, _) ->
+           User_error.raise ~loc
+             [ Pp.textf
+                 "This library has a pullic_name, it already belongs to the \
+                  package %s"
+                 (Package.Name.to_string (Package.name public.package))
              ]
        in
        Option.both virtual_modules implements
@@ -649,15 +714,15 @@ module Library = struct
                 [ Pp.textf "A library cannot be both virtual and implement %s"
                     (Lib_name.to_string impl)
                 ]);
-       ( match (virtual_modules, default_implementation) with
+       (match (virtual_modules, default_implementation) with
        | None, Some (loc, _) ->
          User_error.raise ~loc
            [ Pp.text
                "Only virtual libraries can specify a default implementation."
            ]
-       | _ -> () );
+       | _ -> ());
        { name
-       ; public
+       ; visibility
        ; synopsis
        ; install_c_headers
        ; ppx_runtime_libraries
@@ -683,13 +748,26 @@ module Library = struct
        ; instrumentation_backend
        })
 
+  let package t =
+    match t.visibility with
+    | Public p -> Some p.package
+    | Private p -> p
+
+  let sub_dir t =
+    match t.visibility with
+    | Public p -> p.sub_dir
+    | Private None -> None
+    | Private (Some _) ->
+      Lib_name.Local.mangled_path_under_package (snd t.name)
+      |> String.concat ~sep:"/" |> Option.some
+
   let has_foreign t = Buildable.has_foreign t.buildable
 
   let foreign_archives t =
-    ( if List.is_empty t.buildable.foreign_stubs then
+    (if List.is_empty t.buildable.foreign_stubs then
       []
     else
-      [ Foreign.Archive.stubs (Lib_name.Local.to_string (snd t.name)) ] )
+      [ Foreign.Archive.stubs (Lib_name.Local.to_string (snd t.name)) ])
     @ List.map ~f:snd t.buildable.foreign_archives
 
   let foreign_lib_files t ~dir ~ext_lib =
@@ -700,22 +778,30 @@ module Library = struct
     List.map (foreign_archives t) ~f:(fun archive ->
         Foreign.Archive.dll_file ~archive ~dir ~ext_dll)
 
-  let archive t ~dir ~ext =
-    Path.Build.relative dir (Lib_name.Local.to_string (snd t.name) ^ ext)
+  let archive_basename t ~ext = Lib_name.Local.to_string (snd t.name) ^ ext
+
+  let archive t ~dir ~ext = Path.Build.relative dir (archive_basename t ~ext)
 
   let best_name t =
-    match t.public with
-    | None -> Lib_name.of_local t.name
-    | Some p -> snd p.name
+    match t.visibility with
+    | Private _ -> Lib_name.of_local t.name
+    | Public p -> snd p.name
 
   let is_virtual t = Option.is_some t.virtual_modules
 
   let is_impl t = Option.is_some t.implements
 
   let obj_dir ~dir t =
+    let private_lib =
+      match t.visibility with
+      | Private (Some _) -> true
+      | Private None
+      | Public _ ->
+        false
+    in
     Obj_dir.make_lib ~dir
       ~has_private_modules:(t.private_modules <> None)
-      (snd t.name)
+      ~private_lib (snd t.name)
 
   let main_module_name t : Lib_info.Main_module_name.t =
     match (t.implements, t.wrapped) with
@@ -728,35 +814,56 @@ module Library = struct
       This (Some (Module_name.of_local_lib_name (snd t.name)))
 
   let to_lib_info conf ~dir
-      ~lib_config:({ Lib_config.has_native; ext_lib; ext_dll; _ } as lib_config)
-      =
-    let _loc, lib_name = conf.name in
+      ~lib_config:
+        ({ Lib_config.has_native; ext_lib; ext_dll; natdynlink_supported; _ } as
+        lib_config) =
     let obj_dir = obj_dir ~dir conf in
-    let gen_archive_file ~dir ext =
-      Path.Build.relative dir (Lib_name.Local.to_string lib_name ^ ext)
+    let archive ?(dir = dir) ext = archive conf ~dir ~ext in
+    let modes = Mode_conf.Set.eval ~has_native conf.modes in
+    let archive_for_mode ~f_ext ~mode =
+      if Mode.Dict.get modes mode then
+        Some (archive (f_ext mode))
+      else
+        None
     in
-    let archive_file = gen_archive_file ~dir in
-    let archive_files ~f_ext =
-      Mode.Dict.of_func (fun ~mode -> [ archive_file (f_ext mode) ])
+    let archives_for_mode ~f_ext =
+      Mode.Dict.of_func (fun ~mode ->
+          archive_for_mode ~f_ext ~mode |> Option.to_list)
     in
     let jsoo_runtime =
       List.map conf.buildable.js_of_ocaml.javascript_files
         ~f:(Path.Build.relative dir)
     in
     let status =
-      match conf.public with
-      | None -> Lib_info.Status.Private conf.project
-      | Some p -> Public (Dune_project.name conf.project, p.package)
+      match conf.visibility with
+      | Private pkg -> Lib_info.Status.Private (conf.project, pkg)
+      | Public p -> Public (conf.project, p.package)
     in
     let virtual_library = is_virtual conf in
     let foreign_archives = foreign_lib_files conf ~dir ~ext_lib in
     let native_archives =
-      [ Path.Build.relative dir (Lib_name.Local.to_string lib_name ^ ext_lib) ]
+      let archive = archive ext_lib in
+      if virtual_library || not modes.native then
+        Lib_info.Files []
+      else if
+        Option.is_some conf.implements
+        || Lib_config.linker_can_create_empty_archives lib_config
+           && Ocaml_version.ocamlopt_always_calls_library_linker
+                lib_config.ocaml_version
+      then
+        Lib_info.Files [ archive ]
+      else
+        Lib_info.Needs_module_info archive
     in
     let foreign_dll_files = foreign_dll_files conf ~dir ~ext_dll in
     let exit_module = Option.bind conf.stdlib ~f:(fun x -> x.exit_module) in
     let jsoo_archive =
-      Some (gen_archive_file ~dir:(Obj_dir.obj_dir obj_dir) ".cma.js")
+      (* XXX we shouldn't access the directory of the obj_dir directly. We
+         should use something like [Obj_dir.Archive.obj] instead *)
+      if modes.byte then
+        Some (archive ~dir:(Obj_dir.obj_dir obj_dir) ".cma.js")
+      else
+        None
     in
     let virtual_ =
       Option.map conf.virtual_modules ~f:(fun _ -> Lib_info.Source.Local)
@@ -766,12 +873,22 @@ module Library = struct
       if virtual_library then
         (Mode.Dict.make_both [], Mode.Dict.make_both [])
       else
-        ( archive_files ~f_ext:Mode.compiled_lib_ext
-        , archive_files ~f_ext:Mode.plugin_ext )
+        let plugins =
+          let archive_file ~mode =
+            archive_for_mode ~f_ext:Mode.plugin_ext ~mode |> Option.to_list
+          in
+          { Mode.Dict.native =
+              (if Dynlink_supported.get conf.dynlink natdynlink_supported then
+                archive_file ~mode:Native
+              else
+                [])
+          ; byte = archive_file ~mode:Byte
+          }
+        in
+        (archives_for_mode ~f_ext:Mode.compiled_lib_ext, plugins)
     in
     let main_module_name = main_module_name conf in
     let name = best_name conf in
-    let modes = Mode_conf.Set.eval ~has_native conf.modes in
     let enabled =
       let enabled_if_result =
         Blang.eval conf.enabled_if ~dir:(Path.build dir) ~f:(fun v _ver ->
@@ -793,6 +910,7 @@ module Library = struct
     let version =
       match status with
       | Public (_, pkg) -> pkg.version
+      | Installed_private
       | Installed
       | Private _ ->
         None
@@ -813,18 +931,39 @@ module Library = struct
     let wrapped = Some conf.wrapped in
     let special_builtin_support = conf.special_builtin_support in
     let instrumentation_backend = conf.instrumentation_backend in
-    Lib_info.create ~loc ~name ~kind ~status ~src_dir ~orig_src_dir ~obj_dir
-      ~version ~synopsis ~main_module_name ~sub_systems ~requires
-      ~foreign_objects ~plugins ~archives ~ppx_runtime_deps ~foreign_archives
-      ~native_archives ~foreign_dll_files ~jsoo_runtime ~jsoo_archive
-      ~preprocess ~enabled ~virtual_deps ~dune_version ~virtual_ ~implements
-      ~default_implementation ~modes ~wrapped ~special_builtin_support
-      ~exit_module ~instrumentation_backend
+    let entry_modules = Lib_info.Source.Local in
+    Lib_info.create ~loc ~path_kind:Local ~name ~kind ~status ~src_dir
+      ~orig_src_dir ~obj_dir ~version ~synopsis ~main_module_name ~sub_systems
+      ~requires ~foreign_objects ~plugins ~archives ~ppx_runtime_deps
+      ~foreign_archives ~native_archives ~foreign_dll_files ~jsoo_runtime
+      ~jsoo_archive ~preprocess ~enabled ~virtual_deps ~dune_version ~virtual_
+      ~entry_modules ~implements ~default_implementation ~modes ~wrapped
+      ~special_builtin_support ~exit_module ~instrumentation_backend
+end
+
+module Plugin = struct
+  type t =
+    { package : Package.t
+    ; name : Package.Name.t
+    ; libraries : (Loc.t * Lib_name.t) list
+    ; site : Loc.t * (Package.Name.t * Section.Site.t)
+    ; optional : bool
+    }
+
+  let decode =
+    fields
+      (let+ name = field "name" Package.Name.decode
+       and+ libraries = field "libraries" (repeat (located Lib_name.decode))
+       and+ site =
+         field "site" (located (pair Package.Name.decode Section.Site.decode))
+       and+ package = Stanza_common.Pkg.field ~stanza:"plugin"
+       and+ optional = field_b "optional" in
+       { name; libraries; site; package; optional })
 end
 
 module Install_conf = struct
   type t =
-    { section : Install.Section.t
+    { section : Install.Section_with_site.t
     ; files : File_binding.Unexpanded.t list
     ; package : Package.t
     ; enabled_if : Blang.t
@@ -832,9 +971,9 @@ module Install_conf = struct
 
   let decode =
     fields
-      (let+ section = field "section" Install.Section.decode
+      (let+ section = field "section" Install.Section_with_site.decode
        and+ files = field "files" File_binding.Unexpanded.L.decode
-       and+ package = Pkg.field "install"
+       and+ package = Stanza_common.Pkg.field ~stanza:"install"
        and+ enabled_if =
          let allowed_vars = Enabled_if.common_vars ~since:(2, 6) in
          Enabled_if.decode ~allowed_vars ~since:(Some (2, 6)) ()
@@ -857,14 +996,14 @@ module Promote = struct
            (Dune_lang.Syntax.since Stanza.syntax (1, 10) >>> into_decode)
        and+ only =
          field_o "only"
-           ( Dune_lang.Syntax.since Stanza.syntax (1, 10)
-           >>> Predicate_lang.Glob.decode )
+           (Dune_lang.Syntax.since Stanza.syntax (1, 10)
+           >>> Predicate_lang.Glob.decode)
        in
        { Rule.Promote.lifetime =
-           ( if until_clean then
+           (if until_clean then
              Until_clean
            else
-             Unlimited )
+             Unlimited)
        ; into
        ; only
        })
@@ -918,8 +1057,8 @@ module Executables = struct
 
     let multi_fields =
       map_validate
-        (let+ names = field_o "names" (repeat (located string))
-         and+ pub_names = field_o "public_names" (repeat public_name) in
+        (let+ names = field_o "names" (repeat1 (located string))
+         and+ pub_names = field_o "public_names" (repeat1 public_name) in
          (names, pub_names))
         ~f:(fun (names, public_names) ->
           match (names, public_names) with
@@ -959,7 +1098,7 @@ module Executables = struct
       and+ package =
         field_o "package"
           (let+ loc = loc
-           and+ pkg = Pkg.decode in
+           and+ pkg = Stanza_common.Pkg.decode in
            (loc, pkg))
       and+ project = Dune_project.get_exn () in
       let names, public_names = names in
@@ -994,8 +1133,8 @@ module Executables = struct
                            to be a valid module name or add a \"name\" field \
                            with a valid module name."
                       ]
-                      ~hints:(Module_name.valid_format_doc :: user_message.hints)
-                  ))
+                      ~hints:
+                        (Module_name.valid_format_doc :: user_message.hints)))
           else
             User_error.raise ~loc
               [ Pp.textf "%s field may not be omitted before dune version %s"
@@ -1025,7 +1164,8 @@ module Executables = struct
             Some
               { public_names
               ; package =
-                  Pkg.default_exn ~loc project (pluralize "executable" ~multi)
+                  Stanza_common.Pkg.default_exn ~loc project
+                    (pluralize "executable" ~multi)
               }
         | Some (loc, _), None ->
           User_error.raise ~loc
@@ -1045,7 +1185,7 @@ module Executables = struct
                       ~dst:(locp, pub)))
             |> List.filter_opt
           in
-          { Install_conf.section = Bin; files; package; enabled_if })
+          { Install_conf.section = Section Bin; files; package; enabled_if })
   end
 
   module Link_mode = struct
@@ -1065,7 +1205,7 @@ module Executables = struct
         | Other a, Other b -> (
           match Poly.compare a.mode b.mode with
           | Eq -> Poly.compare a.kind b.kind
-          | ne -> ne )
+          | ne -> ne)
 
       let to_dyn _ = Dyn.opaque
     end
@@ -1166,7 +1306,7 @@ module Executables = struct
         | Byte, Js -> ".bc.js"
         | Native, Js ->
           User_error.raise ~loc
-            [ Pp.text "Javascript generation only supports bytecode!" ] )
+            [ Pp.text "Javascript generation only supports bytecode!" ])
 
     module O = Comparable.Make (T)
 
@@ -1182,10 +1322,10 @@ module Executables = struct
             List.fold_left l ~init:empty ~f:(fun acc (loc, link_mode) ->
                 set acc link_mode loc)
           in
-          ( match
-              String.Map.of_list_map (to_list t) ~f:(fun (lm, loc) ->
-                  (extension lm ~loc ~ext_obj:".OBJ" ~ext_dll:".DLL", lm))
-            with
+          (match
+             String.Map.of_list_map (to_list t) ~f:(fun (lm, loc) ->
+                 (extension lm ~loc ~ext_obj:".OBJ" ~ext_dll:".DLL", lm))
+           with
           | Ok _ -> ()
           | Error (_ext, (lm1, _), (lm2, _)) ->
             User_error.raise ~loc
@@ -1194,7 +1334,7 @@ module Executables = struct
                    the same file extension."
                   (Dune_lang.to_string (encode lm1))
                   (Dune_lang.to_string (encode lm2))
-              ] );
+              ]);
           t
 
       let byte_and_exe = of_list_exn [ (byte, Loc.none); (exe, Loc.none) ]
@@ -1237,7 +1377,7 @@ module Executables = struct
 
   let common =
     let* dune_version = Dune_lang.Syntax.get_exn Stanza.syntax in
-    let+ buildable = Buildable.decode ~in_library:false ~allow_re_export:false
+    let+ buildable = Buildable.decode Executable
     and+ (_ : bool) =
       field "link_executables" ~default:true
         (Dune_lang.Syntax.deleted_in Stanza.syntax (1, 0) >>> bool)
@@ -1266,12 +1406,12 @@ module Executables = struct
                  ]))
     and+ embed_in_plugin_libraries =
       field_o "embed_in_plugin_libraries"
-        ( Dune_lang.Syntax.since Stanza.syntax (2, 4)
-        >>> located (repeat (located Lib_name.decode)) )
+        (Dune_lang.Syntax.since Stanza.syntax (2, 4)
+        >>> located (repeat (located Lib_name.decode)))
     and+ forbidden_libraries =
       field "forbidden_libraries"
-        ( Dune_lang.Syntax.since Stanza.syntax (2, 0)
-        >>> repeat (located Lib_name.decode) )
+        (Dune_lang.Syntax.since Stanza.syntax (2, 0)
+        >>> repeat (located Lib_name.decode))
         ~default:[]
     and+ bootstrap_info =
       field_o "bootstrap_info"
@@ -1301,11 +1441,13 @@ module Executables = struct
         | None when has_public_name ->
           User_error.raise ~loc:buildable.loc
             [ Pp.textf "No installable mode found for %s."
-                ( if multi then
+                (if multi then
                   "these executables"
                 else
-                  "this executable" )
-            ; Pp.text "One of the following modes is required:"
+                  "this executable")
+            ; Pp.text
+                "When public_name is set, one of the following modes is \
+                 required:"
             ; Pp.enumerate Link_mode.installable_modes ~f:(fun mode ->
                   Pp.verbatim (Dune_lang.to_string (Link_mode.encode mode)))
             ]
@@ -1438,6 +1580,7 @@ module Rule = struct
       ; ("fallback", Field)
       ; ("mode", Field)
       ; ("alias", Field)
+      ; ("enabled_if", Field)
       ]
 
   let short_form =
@@ -1478,7 +1621,8 @@ module Rule = struct
       Enabled_if.decode ~allowed_vars:Any ~since:(Some (1, 4)) ()
     and+ package =
       field_o "package"
-        (Dune_lang.Syntax.since Stanza.syntax (2, 0) >>> Pkg.decode)
+        (Dune_lang.Syntax.since Stanza.syntax (2, 0)
+        >>> Stanza_common.Pkg.decode)
     and+ alias =
       field_o "alias"
         (Dune_lang.Syntax.since Stanza.syntax (2, 0) >>> Alias.Name.decode)
@@ -1496,7 +1640,7 @@ module Rule = struct
             (User_message.did_you_mean s
                ~candidates:(String.Map.keys atom_table))
       | Some Field -> fields long_form
-      | Some Action -> short_form )
+      | Some Action -> short_form)
     | sexp ->
       User_error.raise ~loc:(Dune_lang.Ast.loc sexp)
         [ Pp.textf "S-expression of the form (<atom> ...) expected" ]
@@ -1630,7 +1774,7 @@ module Alias_conf = struct
   let decode =
     fields
       (let+ name = field "name" Alias.Name.decode
-       and+ package = field_o "package" Pkg.decode
+       and+ package = field_o "package" Stanza_common.Pkg.decode
        and+ action =
          field_o "action"
            (let extra_info = "Use a rule stanza with the alias field instead" in
@@ -1658,11 +1802,10 @@ module Tests = struct
 
   let gen_parse names =
     fields
-      (let+ buildable =
-         Buildable.decode ~in_library:false ~allow_re_export:false
+      (let+ buildable = Buildable.decode Executable
        and+ link_flags = Ordered_set_lang.Unexpanded.field "link_flags"
        and+ names = names
-       and+ package = field_o "package" Pkg.decode
+       and+ package = field_o "package" Stanza_common.Pkg.decode
        and+ locks = field "locks" (repeat String_with_vars.decode) ~default:[]
        and+ modes =
          field "modes" Executables.Link_mode.Map.decode
@@ -1673,12 +1816,12 @@ module Tests = struct
          Enabled_if.decode ~allowed_vars:Any ~since:(Some (1, 4)) ()
        and+ action =
          field_o "action"
-           ( Dune_lang.Syntax.since ~fatal:false Stanza.syntax (1, 2)
-           >>> Action_dune_lang.decode )
+           (Dune_lang.Syntax.since ~fatal:false Stanza.syntax (1, 2)
+           >>> Action_dune_lang.decode)
        and+ forbidden_libraries =
          field "forbidden_libraries"
-           ( Dune_lang.Syntax.since Stanza.syntax (2, 0)
-           >>> repeat (located Lib_name.decode) )
+           (Dune_lang.Syntax.since Stanza.syntax (2, 0)
+           >>> repeat (located Lib_name.decode))
            ~default:[]
        in
        { exes =
@@ -1703,7 +1846,7 @@ module Tests = struct
        ; action
        })
 
-  let multi = gen_parse (field "names" (repeat (located string)))
+  let multi = gen_parse (field "names" (repeat1 (located string)))
 
   let single = gen_parse (field "name" (located string) >>| List.singleton)
 end
@@ -1746,6 +1889,7 @@ module Copy_files = struct
     { add_line_directive : bool
     ; alias : Alias.Name.t option
     ; mode : Rule.Mode.t
+    ; enabled_if : Blang.t
     ; files : String_with_vars.t
     ; syntax_version : Dune_lang.Syntax.Version.t
     }
@@ -1755,9 +1899,17 @@ module Copy_files = struct
     let+ alias = field_o "alias" (check >>> Alias.Name.decode)
     and+ mode =
       field "mode" ~default:Rule.Mode.Standard (check >>> Rule.Mode.decode)
+    and+ enabled_if =
+      Enabled_if.decode ~allowed_vars:Any ~since:(Some (2, 8)) ()
     and+ files = field "files" (check >>> String_with_vars.decode)
     and+ syntax_version = Dune_lang.Syntax.get_exn Stanza.syntax in
-    { add_line_directive = false; alias; mode; files; syntax_version }
+    { add_line_directive = false
+    ; alias
+    ; mode
+    ; enabled_if
+    ; files
+    ; syntax_version
+    }
 
   let decode =
     peek_exn >>= function
@@ -1768,6 +1920,7 @@ module Copy_files = struct
       { add_line_directive = false
       ; alias = None
       ; mode = Standard
+      ; enabled_if = Blang.true_
       ; files
       ; syntax_version
       }
@@ -1782,7 +1935,7 @@ module Documentation = struct
 
   let decode =
     fields
-      (let+ package = Pkg.field "documentation"
+      (let+ package = Stanza_common.Pkg.field ~stanza:"documentation"
        and+ mld_files = Ordered_set_lang.field "mld_files"
        and+ loc = loc in
        { loc; package; mld_files })
@@ -1820,18 +1973,32 @@ module Library_redirect = struct
   module Local = struct
     type nonrec t = (Loc.t * Lib_name.Local.t) t
 
+    let for_lib (lib : Library.t) ~new_public_name ~loc : t =
+      { loc; new_public_name; old_name = lib.name; project = lib.project }
+
+    let of_private_lib (lib : Library.t) : t option =
+      match lib.visibility with
+      | Public _
+      | Private None ->
+        None
+      | Private (Some package) ->
+        let loc, name = lib.name in
+        let package_name = Package.name package in
+        let new_public_name = (loc, Lib_name.mangled package_name name) in
+        Some (for_lib lib ~loc ~new_public_name)
+
     let of_lib (lib : Library.t) : t option =
       let open Option.O in
-      let* public = lib.public in
-      if Lib_name.equal (Lib_name.of_local lib.name) (snd public.name) then
+      let* public_name =
+        match lib.visibility with
+        | Public plib -> Some plib.name
+        | Private _ -> None
+      in
+      if Lib_name.equal (Lib_name.of_local lib.name) (snd public_name) then
         None
       else
-        Some
-          { loc = Loc.none
-          ; project = lib.project
-          ; old_name = lib.name
-          ; new_public_name = public.name
-          }
+        let loc = fst public_name in
+        Some (for_lib lib ~loc ~new_public_name:public_name)
   end
 end
 
@@ -1850,7 +2017,8 @@ module Deprecated_library_name = struct
           Lib_name.package_name (Public_lib.name public)
         in
         if
-          Package.Name.equal deprecated_package (Public_lib.package public).name
+          let name = Package.name (Public_lib.package public) in
+          Package.Name.equal deprecated_package name
         then
           Not_deprecated
         else
@@ -1871,9 +2039,43 @@ module Deprecated_library_name = struct
        and+ new_public_name =
          field "new_public_name" (located Lib_name.decode)
        in
+       let () =
+         let loc, old_name = (fst old_name).name in
+         if Lib_name.equal (snd new_public_name) old_name then
+           User_error.raise ~loc
+             [ Pp.text
+                 "old_public_name cannot be the same as the new_public_name"
+             ]
+       in
        { Library_redirect.loc; project; old_name; new_public_name })
 end
 
+module Generate_sites_module = struct
+  type t =
+    { loc : Loc.t
+    ; module_ : Module_name.t
+    ; sourceroot : bool
+    ; relocatable : bool
+    ; sites : (Loc.t * Package.Name.t) list
+    ; plugins : (Loc.t * (Package.Name.t * (Loc.t * Section.Site.t))) list
+    }
+
+  let decode =
+    fields
+      (let+ loc = loc
+       and+ module_ = field "module" Module_name.decode
+       and+ sourceroot = field_b "sourceroot"
+       and+ relocatable = field_b "relocatable"
+       and+ sites =
+         field "sites" ~default:[] (repeat (located Package.Name.decode))
+       and+ plugins =
+         field "plugins" ~default:[]
+           (repeat
+              (located (pair Package.Name.decode (located Section.Site.decode))))
+       in
+       { loc; module_; sourceroot; relocatable; sites; plugins })
+end
+
 type Stanza.t +=
   | Library of Library.t
   | Foreign_library of Foreign.Library.t
@@ -1889,6 +2091,8 @@ type Stanza.t +=
   | Library_redirect of Library_redirect.Local.t
   | Deprecated_library_name of Deprecated_library_name.t
   | Cram of Cram_stanza.t
+  | Generate_sites_module of Generate_sites_module.t
+  | Plugin of Plugin.t
 
 module Stanzas = struct
   type t = Stanza.t list
@@ -1908,7 +2112,10 @@ module Stanzas = struct
   let stanzas : constructors =
     [ ( "library"
       , let+ x = Library.decode in
-        [ Library x ] )
+        let base = [ Library x ] in
+        match Library_redirect.Local.of_lib x with
+        | None -> base
+        | Some r -> Library_redirect r :: base )
     ; ( "foreign_library"
       , let+ () = Dune_lang.Syntax.since Stanza.syntax (2, 0)
         and+ x = Foreign.Library.decode in
@@ -1987,6 +2194,14 @@ module Stanzas = struct
       , let+ () = Dune_lang.Syntax.since Stanza.syntax (2, 7)
         and+ t = Cram_stanza.decode in
         [ Cram t ] )
+    ; ( "generate_sites_module"
+      , let+ () = Dune_lang.Syntax.since Section.dune_site_syntax (0, 1)
+        and+ t = Generate_sites_module.decode in
+        [ Generate_sites_module t ] )
+    ; ( "plugin"
+      , let+ () = Dune_lang.Syntax.since Section.dune_site_syntax (0, 1)
+        and+ t = Plugin.decode in
+        [ Plugin t ] )
     ]
 
   let () = Dune_project.Lang.register Stanza.syntax stanzas
@@ -2005,14 +2220,14 @@ module Stanzas = struct
     List.concat_map sexps ~f:(parse stanza_parser)
     |> List.concat_map ~f:(function
          | Include (loc, fn) ->
-           let sexps, context = Include.load_sexps ~context (loc, fn) in
+           let sexps, context = Include_stanza.load_sexps ~context (loc, fn) in
            parse_file_includes ~stanza_parser ~context sexps
          | stanza -> [ stanza ])
 
   let parse ~file (project : Dune_project.t) sexps =
     let stanza_parser = parser project in
     let stanzas =
-      let context = Include.in_file file in
+      let context = Include_stanza.in_file file in
       parse_file_includes ~stanza_parser ~context sexps
     in
     let (_ : bool) =
@@ -2030,10 +2245,11 @@ module Stanzas = struct
 end
 
 let stanza_package = function
-  | Library { public = Some { package; _ }; _ }
+  | Library lib -> Library.package lib
   | Alias { package = Some package; _ }
   | Rule { package = Some package; _ }
   | Install { package; _ }
+  | Plugin { package; _ }
   | Executables { install_conf = Some { package; _ }; _ }
   | Documentation { package; _ }
   | Tests { package = Some package; _ } ->
