@@ -27,6 +27,10 @@ val info : t -> Path.t Lib_info.t
 
 val main_module_name : t -> Module_name.t option Or_exn.t
 
+val entry_module_names : t -> Module_name.t list Or_exn.t
+
+val src_dirs : t -> Path.Set.t
+
 val wrapped : t -> Wrapped.t option Or_exn.t
 
 (** [is_impl lib] returns [true] if the library is an implementation of a
@@ -66,12 +70,16 @@ module L : sig
 
   val to_iflags : Path.Set.t -> _ Command.Args.t
 
-  val include_paths : t -> Path.Set.t
+  val include_paths : ?project:Dune_project.t -> t -> Mode.t -> Path.Set.t
 
-  val include_flags : t -> _ Command.Args.t
+  val include_flags : ?project:Dune_project.t -> t -> Mode.t -> _ Command.Args.t
+
+  val c_include_paths : t -> Path.Set.t
 
   val c_include_flags : t -> _ Command.Args.t
 
+  val toplevel_include_paths : t -> Path.Set.t
+
   val compile_and_link_flags :
     compile:t -> link:t -> mode:Link_mode.t -> _ Command.Args.t
 
@@ -115,6 +123,8 @@ type sub_system = ..
 module Compile : sig
   type t
 
+  type lib
+
   (** Return the list of dependencies needed for linking this library/exe *)
   val requires_link : t -> L.t Or_exn.t Lazy.t
 
@@ -136,9 +146,12 @@ module Compile : sig
 
   val lib_deps_info : t -> Lib_deps_info.t
 
+  val merlin_ident : t -> Merlin_ident.t
+
   (** Sub-systems used in this compilation context *)
   val sub_systems : t -> sub_system list
 end
+with type lib := t
 
 (** {1 Library name resolution} *)
 
@@ -156,6 +169,8 @@ module DB : sig
 
     val not_found : t
 
+    val found : Lib_info.external_ -> t
+
     val to_dyn : t Dyn.Encoder.t
 
     val redirect : db option -> Loc.t * Lib_name.t -> t
@@ -172,27 +187,19 @@ module DB : sig
   val create :
        parent:t option
     -> resolve:(Lib_name.t -> Resolve_result.t)
+    -> projects_by_package:Dune_project.t Package.Name.Map.t
     -> all:(unit -> Lib_name.t list)
+    -> modules_of_lib:(dir:Path.Build.t -> name:Lib_name.t -> Modules.t) Fdecl.t
     -> lib_config:Lib_config.t
     -> unit
     -> t
 
-  module Library_related_stanza : sig
-    type t =
-      | Library of Path.Build.t * Dune_file.Library.t
-      | Library_redirect of Dune_file.Library_redirect.Local.t
-      | Deprecated_library_name of Dune_file.Deprecated_library_name.t
-  end
-
-  (** Create a database from a list of library stanzas *)
-  val create_from_stanzas :
-       parent:t option
-    -> lib_config:Lib_config.t
-    -> Library_related_stanza.t list
+  val create_from_findlib :
+       lib_config:Lib_config.t
+    -> projects_by_package:Dune_project.t Package.Name.Map.t
+    -> Findlib.t
     -> t
 
-  val create_from_findlib : lib_config:Lib_config.t -> Findlib.t -> t
-
   val find : t -> Lib_name.t -> lib option
 
   val find_even_when_hidden : t -> Lib_name.t -> lib option
@@ -205,6 +212,10 @@ module DB : sig
 
   val resolve : t -> Loc.t * Lib_name.t -> lib Or_exn.t
 
+  (** Like [resolve], but will return [None] instead of an error if we are
+      unable to find the library. *)
+  val resolve_when_exists : t -> Loc.t * Lib_name.t -> lib Or_exn.t option
+
   (** Resolve libraries written by the user in a [dune] file. The resulting list
       of libraries is transitively closed and sorted by the order of
       dependencies.
@@ -264,8 +275,6 @@ module Sub_system : sig
     (** Get the instance of the subsystem for this library *)
     val get : lib -> M.t option
   end
-
-  val public_info : lib -> Sub_system_info.t Sub_system_name.Map.t
 end
 with type lib := t
 
@@ -286,6 +295,7 @@ val to_dune_lib :
   -> dir:Path.t
   -> Dune_package.Lib.t Or_exn.t
 
+(** Local libraries *)
 module Local : sig
   type lib
 
