@@ -20,9 +20,11 @@ let package_install_file w pkg =
   match Package.Name.Map.find w.conf.packages pkg with
   | None -> Error ()
   | Some p ->
+    let name = Package.name p in
+    let dir = Package.dir p in
     Ok
-      (Path.Source.relative p.path
-         (Utils.install_file ~package:p.name ~findlib_toolchain:None))
+      (Path.Source.relative dir
+         (Utils.install_file ~package:name ~findlib_toolchain:None))
 
 let setup_env ~capture_outputs =
   let env =
@@ -44,22 +46,7 @@ let scan_workspace ?workspace_file ?x ?(capture_outputs = true) ?profile
     ?instrument_with ~ancestor_vcs () =
   let env = setup_env ~capture_outputs in
   let conf = Dune_load.load ~ancestor_vcs in
-  let () =
-    let path : Path.t option =
-      match workspace_file with
-      | None ->
-        let p = Path.of_string Workspace.filename in
-        Option.some_if (Path.exists p) p
-      | Some p ->
-        if not (Path.exists p) then
-          User_error.raise
-            [ Pp.textf "Workspace file %s does not exist"
-                (Path.to_string_maybe_quoted p)
-            ];
-        Some p
-    in
-    Workspace.init ?x ?profile ?instrument_with ?path ()
-  in
+  let () = Workspace.init ?x ?profile ?instrument_with ?workspace_file () in
   let+ contexts = Context.DB.all () in
   List.iter contexts ~f:(fun (ctx : Context.t) ->
       let open Pp.O in
@@ -71,9 +58,16 @@ let scan_workspace ?workspace_file ?x ?(capture_outputs = true) ?profile
 
 let init_build_system ?only_packages ~sandboxing_preference ?caching w =
   Build_system.reset ();
-  Build_system.init ~sandboxing_preference
-    ~contexts:(List.map ~f:Context.to_build_context w.contexts)
-    ?caching;
+  let promote_source ?chmod ~src ~dst ctx =
+    let conf = Artifact_substitution.conf_of_context ctx in
+    let src = Path.build src in
+    let dst = Path.source dst in
+    Artifact_substitution.copy_file ?chmod ~src ~dst ~conf ()
+  in
+  Build_system.init ~sandboxing_preference ~promote_source
+    ~contexts:(List.map ~f:Context.build_context w.contexts)
+    ?caching ();
+  List.iter w.contexts ~f:Context.init_configurator;
   let+ scontexts = Gen_rules.gen w.conf ~contexts:w.contexts ?only_packages in
   { workspace = w; scontexts }
 
@@ -90,7 +84,7 @@ let auto_concurrency =
           | Some s -> (
             match int_of_string s with
             | exception _ -> Fiber.return 1
-            | n -> Fiber.return n )
+            | n -> Fiber.return n)
         else
           let commands =
             [ ("nproc", [])
@@ -114,7 +108,7 @@ let auto_concurrency =
                 | Ok s -> (
                   match int_of_string (String.trim s) with
                   | n -> Fiber.return n
-                  | exception _ -> loop rest ) ) )
+                  | exception _ -> loop rest)))
           in
           loop commands
       in
@@ -139,3 +133,10 @@ let find_context_exn t ~name =
   | None ->
     User_error.raise
       [ Pp.textf "Context %S not found!" (Context_name.to_string name) ]
+
+let find_scontext_exn t ~name =
+  match Context_name.Map.find t.scontexts name with
+  | Some ctx -> ctx
+  | None ->
+    User_error.raise
+      [ Pp.textf "Context %S not found!" (Context_name.to_string name) ]
