@@ -12,19 +12,24 @@ module Artifacts : sig
   val lookup_module :
     t -> Module_name.t -> (Path.Build.t Obj_dir.t * Module.t) option
 
-  val lookup_library : t -> Lib_name.t -> Dune_file.Library.t option
+  val lookup_library : t -> Lib_name.t -> Lib_info.local option
 end
 
 type t
 
 val artifacts : t -> Artifacts.t
 
-(** Modules attached to a library. [name] is the library best name. *)
-val modules_of_library : t -> name:Lib_name.t -> Modules.t
+type for_ =
+  | Library of Lib_name.t  (** Library name *)
+  | Exe of
+      { first_exe : string
+            (** Name of first executable appearing in executables stanza *)
+      }
 
-(** Modules attached to a set of executables. *)
-val modules_of_executables :
-  t -> obj_dir:Path.Build.t Obj_dir.t -> first_exe:string -> Modules.t
+val modules_and_obj_dir : t -> for_:for_ -> Modules.t * Path.Build.t Obj_dir.t
+
+(** Modules attached to a library or executable.*)
+val modules : t -> for_:for_ -> Modules.t
 
 (** Find out what buildable a module is part of *)
 val lookup_module : t -> Module_name.t -> Dune_file.Buildable.t option
@@ -40,6 +45,7 @@ val empty : t
 
 val make :
      Stanza.t list Dir_with_dune.t
+  -> lib_config:Lib_config.t
   -> loc:Loc.t
   -> lookup_vlib:(dir:Path.Build.t -> t)
   -> include_subdirs:Loc.t * Dune_file.Include_subdirs.t
