@@ -1,5 +1,5 @@
 open! Dune_engine
-open Stdune
+open Import
 module Mode = Wrapped
 
 module Common = struct
@@ -69,6 +69,8 @@ module Stdlib = struct
     let* name = t.exit_module in
     Module_name.Map.find t.modules name
 
+  let exists t ~f = Module_name.Map.exists t.modules ~f
+
   let fold t ~init ~f = Module_name.Map.fold t.modules ~f ~init
 
   let map t ~f = { t with modules = Module_name.Map.map t.modules ~f }
@@ -170,7 +172,7 @@ module Mangle = struct
               (Lib_name.Local.to_string lib)
             |> Module_name.of_string
         ; public = main_module_name
-        } )
+        })
     | Exe ->
       sprintf "dune__exe" |> Module_name.of_string |> Visibility.Map.make_both
 
@@ -221,12 +223,12 @@ module Wrapped = struct
        { main_module_name; modules; wrapped_compat; alias_module; wrapped })
 
   let map
-      ( { modules
-        ; wrapped_compat
-        ; alias_module
-        ; main_module_name = _
-        ; wrapped = _
-        } as t ) ~f =
+      ({ modules
+       ; wrapped_compat
+       ; alias_module
+       ; main_module_name = _
+       ; wrapped = _
+       } as t) ~f =
     { t with
       modules = Module_name.Map.map modules ~f
     ; wrapped_compat = Module_name.Map.map wrapped_compat ~f
@@ -328,6 +330,17 @@ module Wrapped = struct
     let init = Module_name.Map.fold modules ~f ~init in
     Module_name.Map.fold wrapped_compat ~f ~init
 
+  let exists
+      { modules
+      ; wrapped_compat
+      ; alias_module
+      ; main_module_name = _
+      ; wrapped = _
+      } ~f =
+    f alias_module
+    || Module_name.Map.exists modules ~f
+    || Module_name.Map.exists wrapped_compat ~f
+
   let lib_interface t = Module_name.Map.find t.modules t.main_module_name
 
   let find t name =
@@ -344,7 +357,7 @@ module Wrapped = struct
     | Wrapped_compat -> (
       match lib_interface t with
       | Some li -> Option.some_if (name = Module.name li) li
-      | None -> Module_name.Map.find t.modules name )
+      | None -> Module_name.Map.find t.modules name)
     | _ ->
       if is_alias_name t name then
         Some t.alias_module
@@ -410,7 +423,7 @@ module Old_format = struct
        | Simple false -> (
          match as_singleton modules with
          | Some m -> Singleton m
-         | None -> Unwrapped modules )
+         | None -> Unwrapped modules)
        | Yes_with_transition _
        | Simple true -> (
          match (main_module_name, alias_module, as_singleton modules) with
@@ -428,8 +441,7 @@ module Old_format = struct
          | None, _, _
          | _, None, _ ->
            User_error.raise ~loc
-             [ Pp.text "Cannot wrap without main module name or alias module" ]
-         ))
+             [ Pp.text "Cannot wrap without main module name or alias module" ]))
 end
 
 let decode ~version ~src_dir ~implements =
@@ -502,7 +514,7 @@ let lib ~src_dir ~main_module_name ~wrapped ~stdlib ~lib_name ~implements
     | (Yes_with_transition _ | Simple true), Some main_module_name, None ->
       make_wrapped main_module_name
     | (Simple true | Yes_with_transition _), None, _ ->
-      Code_error.raise "Modules.lib: cannot wrap without main module name" [] )
+      Code_error.raise "Modules.lib: cannot wrap without main module name" [])
 
 let impl impl ~vlib =
   match (impl, vlib) with
@@ -523,7 +535,7 @@ let rec find t name =
   | Impl { impl; vlib } -> (
     match find impl name with
     | Some _ as m -> m
-    | None -> find vlib name )
+    | None -> find vlib name)
 
 type from =
   | Vlib
@@ -546,7 +558,7 @@ let rec find_dep t ~of_ name =
         | None ->
           let open Option.O in
           let+ m = find_dep vlib ~of_ name in
-          (Vlib, m) )
+          (Vlib, m))
       | _ -> from_impl_or_lib (find t name)
     in
     match from with
@@ -563,8 +575,8 @@ let exe_unwrapped m = Unwrapped m
 
 let exe_wrapped ~src_dir ~modules =
   match as_singleton modules with
-  | None -> Wrapped (Wrapped.exe ~src_dir ~modules)
   | Some m -> singleton_exe m
+  | None -> Wrapped (Wrapped.exe ~src_dir ~modules)
 
 let rec impl_only = function
   | Stdlib w -> Stdlib.impl_only w
@@ -577,6 +589,18 @@ let rec impl_only = function
   | Wrapped w -> Wrapped.impl_only w
   | Impl { vlib; impl } -> impl_only impl @ impl_only vlib
 
+let rec exists t ~f =
+  match t with
+  | Stdlib w -> Stdlib.exists w ~f
+  | Wrapped m -> Wrapped.exists m ~f
+  | Singleton m -> f m
+  | Unwrapped m -> Module_name.Map.exists m ~f
+  | Impl { vlib; impl } -> exists vlib ~f || exists impl ~f
+
+let has_impl =
+  let has = Module.has ~ml_kind:Impl in
+  exists ~f:has
+
 let rec fold_no_vlib t ~init ~f =
   match t with
   | Stdlib w -> Stdlib.fold w ~init ~f
@@ -642,12 +666,12 @@ let rec map_user_written t ~f =
   | Unwrapped m -> Unwrapped (Module_name.Map.map m ~f)
   | Stdlib w -> Stdlib (Stdlib.map w ~f)
   | Wrapped
-      ( { modules
-        ; alias_module = _
-        ; main_module_name = _
-        ; wrapped_compat = _
-        ; wrapped = _
-        } as w ) ->
+      ({ modules
+       ; alias_module = _
+       ; main_module_name = _
+       ; wrapped_compat = _
+       ; wrapped = _
+       } as w) ->
     let modules = Module_name.Map.map modules ~f in
     Wrapped { w with modules }
   | Impl t -> Impl { t with vlib = map_user_written t.vlib ~f }
@@ -697,7 +721,7 @@ let rec obj_map : 'a. t -> f:(Sourced_module.t -> 'a) -> 'a Module.Obj_map.t =
 let entry_modules t =
   List.filter
     ~f:(fun m -> Module.visibility m = Public)
-    ( match t with
+    (match t with
     | Stdlib w -> Stdlib.lib_interface w |> Option.to_list
     | Singleton m -> [ m ]
     | Unwrapped m -> Module_name.Map.values m
@@ -705,10 +729,10 @@ let entry_modules t =
       (* we assume this is never called for implementations *)
       match Wrapped.lib_interface m with
       | Some m -> [ m ]
-      | None -> [ m.alias_module ] )
+      | None -> [ m.alias_module ])
     | Impl i ->
       Code_error.raise "entry_modules: not defined for implementations"
-        [ ("impl", dyn_of_impl i) ] )
+        [ ("impl", dyn_of_impl i) ])
 
 let virtual_module_names =
   fold_no_vlib ~init:Module_name.Set.empty ~f:(fun m acc ->
@@ -733,13 +757,16 @@ let rec wrapped = function
   | Impl { vlib = _; impl } -> wrapped impl
 
 let rec alias_for t m =
-  match t with
-  | Singleton _
-  | Unwrapped _ ->
-    None
-  | Wrapped w -> Wrapped.alias_for w m
-  | Stdlib w -> Stdlib.alias_for w m
-  | Impl { impl; vlib = _ } -> alias_for impl m
+  match Module.kind m with
+  | Root -> None
+  | _ -> (
+    match t with
+    | Singleton _
+    | Unwrapped _ ->
+      None
+    | Wrapped w -> Wrapped.alias_for w m
+    | Stdlib w -> Stdlib.alias_for w m
+    | Impl { impl; vlib = _ } -> alias_for impl m)
 
 let is_stdlib_alias t m =
   match t with
@@ -763,6 +790,10 @@ let is_empty = function
   | Unwrapped w -> Module_name.Map.is_empty w
   | Wrapped w -> Wrapped.empty w
 
+let as_singleton = function
+  | Singleton m -> Some m
+  | _ -> None
+
 let source_dirs =
   fold_user_written ~init:Path.Set.empty ~f:(fun m acc ->
       Module.sources m
