@@ -1,7 +1,7 @@
 (** Module layout information. Contains information about aliasing, wrapping. *)
-open! Dune_engine
 
-open Stdune
+open! Dune_engine
+open Import
 
 type t
 
@@ -100,4 +100,9 @@ val relocate_alias_module : t -> src_dir:Path.t -> t
 
 val is_empty : t -> bool
 
+val as_singleton : t -> Module.t option
+
 val source_dirs : t -> Path.Set.t
+
+(** [has_impl t] is true if there's at least one implementation in [t]*)
+val has_impl : t -> bool
