@@ -5,3 +5,7 @@ let library_path = None
 let library_destdir = None
 
 let mandir = None
+
+let docdir = None
+
+let etcdir = None
