@@ -2,13 +2,24 @@ open! Dune_engine
 open! Stdune
 open Import
 
-let default_context_flags (ctx : Context.t) =
-  (* TODO DUNE3 To ensure full backward compatibility, ocaml_cflags are still
-     present in the :standard set of flags. However these should not as they are
-     already prepended when calling the compiler, causing flag duplication. *)
-  let c = Ocaml_config.ocamlc_cflags ctx.ocaml_config in
-  let cxx =
-    List.filter c ~f:(fun s -> not (String.is_prefix s ~prefix:"-std="))
+let default_context_flags (ctx : Context.t) ~project =
+  let cflags = Ocaml_config.ocamlc_cflags ctx.ocaml_config in
+  let cxxflags =
+    List.filter cflags ~f:(fun s -> not (String.is_prefix s ~prefix:"-std="))
+  in
+  let c, cxx =
+    match Dune_project.use_standard_c_and_cxx_flags project with
+    | None
+    | Some false ->
+      (Build.return cflags, Build.return cxxflags)
+    | Some true ->
+      let c = cflags @ Ocaml_config.ocamlc_cppflags ctx.ocaml_config in
+      let cxx =
+        let open Build.O in
+        let+ db_flags = Cxx_flags.get_flags ctx.build_dir in
+        db_flags @ cxxflags
+      in
+      (Build.return c, cxx)
   in
   Foreign_language.Dict.make ~c ~cxx
 
@@ -17,6 +28,8 @@ module Env_tree : sig
 
   val get_node : t -> dir:Path.Build.t -> Env_node.t
 
+  val get_context_env : t -> Env.t
+
   val create :
        context:Context.t
     -> host_env_tree:t option
@@ -25,6 +38,7 @@ module Env_tree : sig
     -> stanzas_per_dir:Stanza.t list Dir_with_dune.t Path.Build.Map.t
     -> root_expander:Expander.t
     -> bin_artifacts:Artifacts.Bin.t
+    -> context_env:Env.t
     -> t
 
   val bin_artifacts_host : t -> dir:Path.Build.t -> Artifacts.Bin.t
@@ -33,6 +47,7 @@ module Env_tree : sig
 end = struct
   type t =
     { context : Context.t
+    ; context_env : Env.t  (** context env with additional variables *)
     ; scopes : Scope.DB.t
     ; default_env : Env_node.t Memo.Lazy.t
     ; stanzas_per_dir : Stanza.t list Dir_with_dune.t Path.Build.Map.t
@@ -44,6 +59,8 @@ end = struct
 
   let get_node t ~dir = t.get_node dir
 
+  let get_context_env t = t.context_env
+
   let bin_artifacts_host t ~dir =
     let bin_artifacts t ~dir = get_node t ~dir |> Env_node.bin_artifacts in
     match t.host with
@@ -100,7 +117,10 @@ end = struct
     let scope = Scope.DB.find_by_dir t.scopes dir in
     let inherit_from =
       if Path.Build.equal dir (Scope.root scope) then
-        t.default_env
+        let format_config = Dune_project.format_config (Scope.project scope) in
+        Memo.lazy_ (fun () ->
+            let default_env = Memo.Lazy.force t.default_env in
+            Env_node.set_format_config default_env format_config)
       else
         match Path.Build.parent dir with
         | None ->
@@ -109,7 +129,8 @@ end = struct
         | Some parent -> Memo.lazy_ (fun () -> get_node t ~dir:parent)
     in
     let config_stanza = get_env_stanza t ~dir in
-    let default_context_flags = default_context_flags t.context in
+    let project = Scope.project scope in
+    let default_context_flags = default_context_flags t.context ~project in
     let expander_for_artifacts =
       Memo.lazy_ (fun () ->
           expander_for_artifacts ~scope ~root_expander:t.root_expander
@@ -121,7 +142,7 @@ end = struct
     in
     Env_node.make ~dir ~scope ~config_stanza ~inherit_from:(Some inherit_from)
       ~profile:t.context.profile ~expander ~expander_for_artifacts
-      ~default_context_flags ~default_env:t.context.env
+      ~default_context_flags ~default_env:t.context_env
       ~default_bin_artifacts:t.bin_artifacts
 
   (* Here we jump through some hoops to construct [t] as well as create a
@@ -136,7 +157,7 @@ end = struct
      recursive module [Rec]. Since recursive let-modules are not allowed either,
      we need to also wrap [Rec] inside a non-recursive module [Non_rec]. *)
   let create ~context ~host_env_tree ~scopes ~default_env ~stanzas_per_dir
-      ~root_expander ~bin_artifacts =
+      ~root_expander ~bin_artifacts ~context_env =
     let module Non_rec = struct
       module rec Rec : sig
         val env_tree : unit -> t
@@ -145,6 +166,7 @@ end = struct
       end = struct
         let env_tree =
           { context
+          ; context_env
           ; scopes
           ; default_env
           ; stanzas_per_dir
@@ -199,6 +221,8 @@ type t =
 
 let context t = t.context
 
+let context_env t = Env_tree.get_context_env t.env_tree
+
 let stanzas t = t.stanzas
 
 let stanzas_in t ~dir = Path.Build.Map.find t.stanzas_per_dir dir
@@ -215,6 +239,31 @@ let to_dyn t = Context.to_dyn t.context
 
 let host t = Option.value t.host ~default:t
 
+let get_site_of_packages_aux ~loc ~packages ~context ~pkg ~site =
+  let find_site sites ~pkg ~site =
+    match Section.Site.Map.find sites site with
+    | Some section -> section
+    | None ->
+      User_error.raise ~loc
+        [ Pp.textf "Package %s doesn't define a site %s"
+            (Package.Name.to_string pkg)
+            (Section.Site.to_string site)
+        ]
+  in
+  match Package.Name.Map.find packages pkg with
+  | Some p -> find_site p.Package.sites ~pkg ~site
+  | None -> (
+    match Findlib.find_root_package context.Context.findlib pkg with
+    | Ok p -> find_site p.sites ~pkg ~site
+    | Error Not_found ->
+      User_error.raise ~loc
+        [ Pp.textf "The package %s is not found" (Package.Name.to_string pkg) ]
+    | Error (Invalid_dune_package exn) -> Exn.raise exn)
+
+let get_site_of_packages t ~loc ~pkg ~site =
+  get_site_of_packages_aux ~loc ~packages:t.packages ~context:t.context ~pkg
+    ~site
+
 let lib_entries_of_package t pkg_name =
   Package.Name.Map.find t.lib_entries_by_package pkg_name
   |> Option.value ~default:[]
@@ -222,14 +271,13 @@ let lib_entries_of_package t pkg_name =
 let internal_lib_names t =
   List.fold_left t.stanzas ~init:Lib_name.Set.empty
     ~f:(fun acc { Dir_with_dune.data = stanzas; _ } ->
-      List.fold_left stanzas ~init:acc ~f:(fun acc ->
-        function
+      List.fold_left stanzas ~init:acc ~f:(fun acc -> function
         | Dune_file.Library lib ->
           Lib_name.Set.add
-            ( match lib.public with
-            | None -> acc
-            | Some public ->
-              Lib_name.Set.add acc (Dune_file.Public_lib.name public) )
+            (match lib.visibility with
+            | Private _ -> acc
+            | Public public ->
+              Lib_name.Set.add acc (Dune_file.Public_lib.name public))
             (Lib_name.of_local lib.name)
         | _ -> acc))
 
@@ -257,7 +305,7 @@ let make_rule t ?sandbox ?mode ?locks ?loc ~dir build =
   let build = chdir_to_build_context_root t build in
   let env = get_node t.env_tree ~dir |> Env_node.external_env in
   Rule.make ?sandbox ?mode ?locks ~info:(Rule.Info.of_loc_opt loc)
-    ~context:(Some (Context.to_build_context t.context))
+    ~context:(Some (Context.build_context t.context))
     ~env:(Some env) build
 
 let add_rule t ?sandbox ?mode ?locks ?loc ~dir build =
@@ -275,22 +323,18 @@ let add_rules t ?sandbox ~dir builds =
 let add_alias_action t alias ~dir ~loc ?locks ~stamp action =
   let env = Some (get_node t.env_tree ~dir |> Env_node.external_env) in
   Rules.Produce.Alias.add_action
-    ~context:(Context.to_build_context t.context)
+    ~context:(Context.build_context t.context)
     ~env alias ~loc ?locks ~stamp action
 
 let build_dir_is_vendored build_dir =
-  let opt =
-    let open Option.O in
-    let* src_dir = Path.Build.drop_build_context build_dir in
-    let+ src_dir = File_tree.find_dir src_dir in
-    Sub_dirs.Status.Vendored = File_tree.Dir.status src_dir
-  in
-  Option.value ~default:false opt
+  match Path.Build.drop_build_context build_dir with
+  | Some src_dir -> Dune_engine.File_tree.is_vendored src_dir
+  | None -> false
 
-let ocaml_flags t ~dir (x : Dune_file.Buildable.t) =
+let ocaml_flags t ~dir (spec : Ocaml_flags.Spec.t) =
   let expander = Env_tree.expander t.env_tree ~dir in
   let flags =
-    Ocaml_flags.make ~spec:x.flags
+    Ocaml_flags.make ~spec
       ~default:(get_node t.env_tree ~dir |> Env_node.ocaml_flags)
       ~eval:(Expander.expand_and_eval_set expander)
   in
@@ -323,6 +367,8 @@ let odoc t ~dir = get_node t.env_tree ~dir |> Env_node.odoc
 
 let coq t ~dir = get_node t.env_tree ~dir |> Env_node.coq
 
+let format_config t ~dir = get_node t.env_tree ~dir |> Env_node.format_config
+
 let dump_env t ~dir =
   let t = t.env_tree in
   let open Build.O in
@@ -393,10 +439,11 @@ let get_installed_binaries stanzas ~(context : Context.t) =
               acc)
       in
       match (stanza : Stanza.t) with
-      | Dune_file.Install { section = Bin; files; _ } ->
+      | Dune_file.Install { section = Section Bin; files; _ } ->
         binaries_from_install files
       | Dune_file.Executables
-          ({ install_conf = Some { section = Bin; files; _ }; _ } as exes) ->
+          ({ install_conf = Some { section = Section Bin; files; _ }; _ } as
+          exes) ->
         let compile_info =
           let project = Scope.project d.scope in
           let dune_version = Dune_project.dune_version project in
@@ -421,11 +468,61 @@ let get_installed_binaries stanzas ~(context : Context.t) =
           acc
       | _ -> acc)
 
-let create ~(context : Context.t) ?host ~projects ~packages ~stanzas =
+let create_lib_entries_by_package ~public_libs stanzas =
+  Dir_with_dune.deep_fold stanzas ~init:[] ~f:(fun d stanza acc ->
+      match stanza with
+      | Dune_file.Library ({ visibility = Private (Some pkg); _ } as lib) -> (
+        match
+          let db = Scope.libs d.scope in
+          Lib.DB.find db (Dune_file.Library.best_name lib)
+        with
+        | None -> acc
+        | Some lib ->
+          let name = Package.name pkg in
+          (name, Lib_entry.Library (Lib.Local.of_lib_exn lib)) :: acc)
+      | Dune_file.Library { visibility = Public pub; _ } -> (
+        match Lib.DB.find public_libs (Dune_file.Public_lib.name pub) with
+        | None ->
+          (* Skip hidden or unavailable libraries. TODO we should assert that
+             the libary name is always found somehow *)
+          acc
+        | Some lib ->
+          let package = Dune_file.Public_lib.package pub in
+          let name = Package.name package in
+          (name, Lib_entry.Library (Lib.Local.of_lib_exn lib)) :: acc)
+      | Dune_file.Deprecated_library_name
+          ({ old_name = old_public_name, _; _ } as d) ->
+        let package = Dune_file.Public_lib.package old_public_name in
+        let name = Package.name package in
+        (name, Lib_entry.Deprecated_library_name d) :: acc
+      | _ -> acc)
+  |> Package.Name.Map.of_list_multi
+  |> Package.Name.Map.map
+       ~f:
+         (List.sort ~compare:(fun a b ->
+              Lib_name.compare (Lib_entry.name a) (Lib_entry.name b)))
+
+let create_projects_by_package projects : Dune_project.t Package.Name.Map.t =
+  List.concat_map projects ~f:(fun project ->
+      Dune_project.packages project
+      |> Package.Name.Map.values
+      |> List.map ~f:(fun (pkg : Package.t) ->
+             let name = Package.name pkg in
+             (name, project)))
+  |> Package.Name.Map.of_list_exn
+
+let modules_of_lib = Fdecl.create Dyn.Encoder.opaque
+
+let create ~(context : Context.t) ?host ~projects ~packages ~stanzas () =
   let lib_config = Context.lib_config context in
-  let installed_libs = Lib.DB.create_from_findlib context.findlib ~lib_config in
+  let projects_by_package = create_projects_by_package projects in
+  let installed_libs =
+    Lib.DB.create_from_findlib context.findlib ~lib_config ~projects_by_package
+  in
+  let modules_of_lib_for_scope = Fdecl.create Dyn.Encoder.opaque in
   let scopes, public_libs =
-    Scope.DB.create_from_stanzas ~projects ~context ~installed_libs stanzas
+    Scope.DB.create_from_stanzas ~projects ~projects_by_package ~context
+      ~installed_libs ~modules_of_lib:modules_of_lib_for_scope stanzas
   in
   let stanzas =
     List.map stanzas ~f:(fun { Dune_load.Dune_file.dir; project; stanzas } ->
@@ -447,23 +544,95 @@ let create ~(context : Context.t) ?host ~projects ~packages ~stanzas =
     Artifacts.create context ~public_libs ~local_bins
   in
   let root_expander =
-    let artifacts_host =
+    let scopes_host, artifacts_host, context_host =
       match host with
-      | None -> artifacts
-      | Some host -> host.artifacts
+      | None -> (scopes, artifacts, context)
+      | Some host -> (host.scopes, host.artifacts, host.context)
     in
     let find_package = Package.Name.Map.find packages in
     Expander.make
       ~scope:(Scope.DB.find_by_dir scopes context.build_dir)
+      ~scope_host:(Scope.DB.find_by_dir scopes_host context_host.build_dir)
       ~context ~lib_artifacts:artifacts.public_libs
-      ~bin_artifacts_host:artifacts_host.bin ~find_package
+      ~bin_artifacts_host:artifacts_host.bin
+      ~lib_artifacts_host:artifacts_host.public_libs ~find_package
   in
+  let dune_dir_locations_var : Stdune.Env.Var.t = "DUNE_DIR_LOCATIONS" in
+  (* Add the section of the site mentioned in stanzas (it could be a site of an
+     external package) *)
+  let add_in_package_section m pkg section =
+    Package.Name.Map.update m pkg ~f:(function
+      | None -> Some (Section.Set.singleton section)
+      | Some s -> Some (Section.Set.add s section))
+  in
+  let package_sections =
+    Dir_with_dune.deep_fold stanzas ~init:Package.Name.Map.empty
+      ~f:(fun _ stanza acc ->
+        let add_in_package_sites acc pkg site loc =
+          let section =
+            get_site_of_packages_aux ~loc ~packages ~context ~pkg ~site
+          in
+          add_in_package_section acc pkg section
+        in
+        match stanza with
+        | Dune_file.Install { section = Site { pkg; site; loc }; _ } ->
+          add_in_package_sites acc pkg site loc
+        | Dune_file.Plugin { site = loc, (pkg, site); _ } ->
+          add_in_package_sites acc pkg site loc
+        | _ -> acc)
+  in
+  (* Add the site of the local package: it should only useful for making sure
+     that at least one location is given to the site of local package because if
+     the site is used it should already be in [packages_sections] *)
+  let package_sections =
+    Package.Name.Map.foldi ~init:package_sections packages
+      ~f:(fun package_name package acc ->
+        Section.Site.Map.fold ~init:acc package.Package.sites
+          ~f:(fun section acc ->
+            add_in_package_section acc package_name section))
+  in
+  let env_dune_dir_locations =
+    let install_dir = Config.local_install_dir ~context:context.Context.name in
+    let install_dir = Path.build install_dir in
+    let v =
+      Option.value
+        (Stdune.Env.get context.env dune_dir_locations_var)
+        ~default:""
+    in
+    Package.Name.Map.foldi ~init:v package_sections
+      ~f:(fun package_name sections init ->
+        let paths =
+          Install.Section.Paths.make ~package:package_name ~destdir:install_dir
+            ()
+        in
+        Section.Set.fold sections ~init ~f:(fun section acc ->
+            sprintf "%s%c%s%c%s%s"
+              (Package.Name.to_string package_name)
+              Stdune.Bin.path_sep
+              (Section.to_string section)
+              Stdune.Bin.path_sep
+              (Path.to_absolute_filename
+                 (Install.Section.Paths.get paths section))
+              (if String.is_empty acc then
+                acc
+              else
+                sprintf "%c%s" Stdune.Bin.path_sep acc)))
+  in
+  let context_env =
+    if String.is_empty env_dune_dir_locations then
+      context.env
+    else
+      Stdune.Env.add context.env ~var:dune_dir_locations_var
+        ~value:env_dune_dir_locations
+  in
+
   let default_env =
     Memo.lazy_ (fun () ->
         let make ~inherit_from ~config_stanza =
           let dir = context.build_dir in
           let scope = Scope.DB.find_by_dir scopes dir in
-          let default_context_flags = default_context_flags context in
+          let project = Scope.project scope in
+          let default_context_flags = default_context_flags context ~project in
           let expander_for_artifacts =
             Memo.lazy_ (fun () ->
                 Code_error.raise
@@ -472,7 +641,7 @@ let create ~(context : Context.t) ?host ~projects ~packages ~stanzas =
           let expander = Memo.Lazy.of_val root_expander in
           Env_node.make ~dir ~scope ~inherit_from ~config_stanza
             ~profile:context.profile ~expander ~expander_for_artifacts
-            ~default_context_flags ~default_env:context.env
+            ~default_context_flags ~default_env:context_env
             ~default_bin_artifacts:artifacts.bin
         in
         make ~config_stanza:context.env_nodes.context
@@ -485,50 +654,37 @@ let create ~(context : Context.t) ?host ~projects ~packages ~stanzas =
   let env_tree =
     Env_tree.create ~context ~scopes ~default_env ~stanzas_per_dir
       ~host_env_tree:(Option.map host ~f:(fun x -> x.env_tree))
-      ~root_expander ~bin_artifacts:artifacts.bin
+      ~root_expander ~bin_artifacts:artifacts.bin ~context_env
   in
   let dir_status_db = Dir_status.DB.make ~stanzas_per_dir in
   let projects_by_key =
     Dune_project.File_key.Map.of_list_map_exn projects ~f:(fun project ->
         (Dune_project.file_key project, project))
   in
-  { context
-  ; root_expander
-  ; host
-  ; scopes
-  ; public_libs
-  ; installed_libs
-  ; stanzas
-  ; stanzas_per_dir
-  ; packages
-  ; artifacts
-  ; lib_entries_by_package =
-      Dir_with_dune.deep_fold stanzas ~init:[] ~f:(fun _ stanza acc ->
-          match stanza with
-          | Dune_file.Library { public = Some pub; _ } -> (
-            match Lib.DB.find public_libs (Dune_file.Public_lib.name pub) with
-            | None -> acc
-            | Some lib ->
-              ( (Dune_file.Public_lib.package pub).name
-              , Lib_entry.Library (Option.value_exn (Lib.Local.of_lib lib)) )
-              :: acc )
-          | Dune_file.Deprecated_library_name
-              ({ old_name = old_public_name, deprecated; _ } as d) ->
-            ( (Dune_file.Public_lib.package old_public_name).name
-            , Lib_entry.Deprecated_library_name
-                { d with old_name = (old_public_name, deprecated) } )
-            :: acc
-          | _ -> acc)
-      |> Package.Name.Map.of_list_multi
-      |> Package.Name.Map.map
-           ~f:
-             (List.sort ~compare:(fun a b ->
-                  Lib_name.compare (Lib_entry.name a) (Lib_entry.name b)))
-  ; env_tree
-  ; default_env
-  ; dir_status_db
-  ; projects_by_key
-  }
+  let lib_entries_by_package =
+    create_lib_entries_by_package ~public_libs stanzas
+  in
+  let t =
+    { context
+    ; root_expander
+    ; host
+    ; scopes
+    ; public_libs
+    ; installed_libs
+    ; stanzas
+    ; stanzas_per_dir
+    ; packages
+    ; artifacts
+    ; lib_entries_by_package
+    ; env_tree
+    ; default_env
+    ; dir_status_db
+    ; projects_by_key
+    }
+  in
+  Fdecl.set modules_of_lib_for_scope (fun ~dir ~name ->
+      Fdecl.get modules_of_lib t ~dir ~name);
+  t
 
 let dir_status_db t = t.dir_status_db
 
