@@ -36,8 +36,8 @@ module Common = struct
 
       let more =
         List.rev
-          ( [ field_of_list [ of_string "language"; of_string lang ] ]
-          |> add_more "names" names |> add_more "flags" flags )
+          ([ field_of_list [ of_string "language"; of_string lang ] ]
+          |> add_more "names" names |> add_more "flags" flags)
       in
 
       field_of_list [ of_string "foreign_stubs" ] ~more
@@ -131,10 +131,11 @@ module Common = struct
     iter path;
     !files
 
-  let string_of_sexps sexps comments =
+  let string_of_sexps ~version sexps comments =
     let new_csts = List.map sexps ~f:Dune_lang.Cst.concrete in
     Dune_lang.Parser.insert_comments new_csts comments
-    |> Format.asprintf "%a@?" Format_dune_lang.pp_top_sexps
+    |> Format_dune_lang.pp_top_sexps ~version
+    |> Format.asprintf "%a@?" Pp.to_fmt
 end
 
 module V1 = struct
@@ -199,7 +200,7 @@ module V1 = struct
         match s with
         | "files_recursively_in" ->
           Atom (loc, Dune_lang.Atom.of_string "source_tree")
-        | _ -> upgrade_string s ~loc ~quoted:false )
+        | _ -> upgrade_string s ~loc ~quoted:false)
       | Template _ as x -> x
       | Quoted_string (loc, s) -> upgrade_string s ~loc ~quoted:true
       | List (loc, l) ->
@@ -226,19 +227,21 @@ module V1 = struct
             List.map l ~f:upgrade
           | (Atom (_, A ("preprocess" | "lint")) as field) :: rest ->
             upgrade field
-            :: List.map rest ~f:(fun x ->
-                   map_var (upgrade x) ~f:(fun (v : Dune_lang.Template.var) ->
-                       Dune_lang.Template.Var
-                         ( if v.name = "<" then
-                           { v with name = "input-file" }
-                         else
-                           v )))
+            ::
+            List.map rest ~f:(fun x ->
+                map_var (upgrade x) ~f:(fun (v : Dune_lang.Template.var) ->
+                    Dune_lang.Template.Var
+                      (if v.name = "<" then
+                        { v with name = "input-file" }
+                      else
+                        v)))
           | (Atom (_, A "per_module") as field) :: specs ->
             upgrade field
-            :: List.map specs ~f:(function
-                 | List (loc, [ spec; List (_, modules) ]) ->
-                   List (loc, upgrade spec :: List.map modules ~f:upgrade)
-                 | sexp -> upgrade sexp)
+            ::
+            List.map specs ~f:(function
+              | List (loc, [ spec; List (_, modules) ]) ->
+                List (loc, upgrade spec :: List.map modules ~f:upgrade)
+              | sexp -> upgrade sexp)
           | [ (Atom (_, A "pps") as field); List (_, pps) ] -> (
             let pps, args =
               List.partition_map pps ~f:(function
@@ -249,11 +252,11 @@ module V1 = struct
                 | sexp -> Left sexp)
             in
             let args = List.concat args in
-            (upgrade field :: pps)
+            upgrade field :: pps
             @
             match args with
             | [] -> []
-            | _ -> Atom (loc, Dune_lang.Atom.of_string "--") :: args )
+            | _ -> Atom (loc, Dune_lang.Atom.of_string "--") :: args)
           | [ (Atom (_, A field_name) as field); List (_, args) ]
             when match (field_name, args) with
                  | "rule", Atom (_, A field_name) :: _ ->
@@ -269,11 +272,11 @@ module V1 = struct
                 List
                   ( loc
                   , field
-                    :: (let loc = Dune_lang.Ast.loc first in
-                        List
-                          ( loc
-                          , [ Atom (loc, Dune_lang.Atom.of_string ":<"); first ]
-                          ))
+                    ::
+                    (let loc = Dune_lang.Ast.loc first in
+                     List
+                       ( loc
+                       , [ Atom (loc, Dune_lang.Atom.of_string ":<"); first ] ))
                     :: rest )
               | x -> x)
           else
@@ -311,10 +314,11 @@ module V1 = struct
         let stanza =
           Dune_lang.Ast.add_loc ~loc:Loc.none
             (List
-               ( Dune_lang.atom "data_only_dirs"
-               :: List.map
-                    (String.Set.to_list data_only_dirs)
-                    ~f:Dune_lang.atom_or_quoted_string ))
+               (Dune_lang.atom "data_only_dirs"
+                ::
+                List.map
+                  (String.Set.to_list data_only_dirs)
+                  ~f:Dune_lang.atom_or_quoted_string))
         in
         let sexps = stanza :: sexps in
         (sexps, [ jbuild_ignore ])
@@ -324,7 +328,11 @@ module V1 = struct
         (List.map ~f:Dune_lang.Cst.concrete sexps)
         comments
     in
-    let contents = Format.asprintf "%a@?" Format_dune_lang.pp_top_sexps sexps in
+    let contents =
+      let version = !Dune_project.default_dune_language_version in
+      Format.asprintf "%a@?" Pp.to_fmt
+        (Format_dune_lang.pp_top_sexps ~version sexps)
+    in
     todo.to_rename_and_edit <-
       { original_file = file; new_file; extra_files_to_delete; contents }
       :: todo.to_rename_and_edit
@@ -383,8 +391,8 @@ module V1 = struct
         if start < stop then add_subst start stop ""
       | List
           ( _
-          , ( String (jpos, "jbuilder")
-              :: String (arg_pos, ("build" | "runtest")) :: _ as l ) ) ->
+          , (String (jpos, "jbuilder")
+             :: String (arg_pos, ("build" | "runtest")) :: _ as l) ) ->
         replace_jbuilder jpos;
         let _, _, start = arg_pos in
         let stop = end_offset_of_opam_value (List.last l |> Option.value_exn) in
@@ -445,13 +453,13 @@ module V1 = struct
     Dune_project.default_dune_language_version := (1, 0);
     let project = File_tree.Dir.project dir in
     let project_root = Dune_project.root project in
-    ( if project_root = File_tree.Dir.path dir then
+    (if project_root = File_tree.Dir.path dir then
       let (_ : Dune_project.created_or_already_exist) =
         Dune_project.ensure_project_file_exists project
       in
       Package.Name.Map.iter (Dune_project.packages project) ~f:(fun pkg ->
           let fn = Package.opam_file pkg in
-          if Path.exists (Path.source fn) then upgrade_opam_file todo fn) );
+          if Path.exists (Path.source fn) then upgrade_opam_file todo fn));
     if String.Set.mem (File_tree.Dir.files dir) File_tree.Dune_file.jbuild_fname
     then
       let fn =
@@ -560,14 +568,15 @@ module V2 = struct
     let sexps, comments = read_and_parse (Dune_project.file project) in
     let v = !Dune_project.default_dune_language_version in
     let sexps = sexps |> Ast_tools.bump_lang_version v |> update_formatting in
-    let new_file_contents = string_of_sexps sexps comments in
+    let new_file_contents = string_of_sexps ~version:v sexps comments in
     (* Printf.eprintf "AAAH: %b\n!" (was_formatted sexps); *)
     todo.to_edit <-
       (Dune_project.file project, new_file_contents) :: todo.to_edit
 
   let upgrade_dune_file todo fn sexps comments =
     let new_ast = sexps |> List.map ~f:update_stanza in
-    let new_file_contents = string_of_sexps new_ast comments in
+    let version = !Dune_project.default_dune_language_version in
+    let new_file_contents = string_of_sexps ~version new_ast comments in
     todo.to_edit <- (fn, new_file_contents) :: todo.to_edit
 
   let upgrade_dune_files todo dir =
@@ -680,10 +689,10 @@ let upgrade () =
         let { original_file; new_file; extra_files_to_delete; contents } = x in
         Console.print
           [ Pp.textf "Upgrading %s to %s..."
-              ( List.map
-                  (extra_files_to_delete @ [ original_file ])
-                  ~f:Path.Source.to_string_maybe_quoted
-              |> String.enumerate_and )
+              (List.map
+                 (extra_files_to_delete @ [ original_file ])
+                 ~f:Path.Source.to_string_maybe_quoted
+              |> String.enumerate_and)
               (Path.Source.to_string_maybe_quoted new_file)
           ];
         List.iter (original_file :: extra_files_to_delete) ~f:(fun p ->
