@@ -30,6 +30,7 @@ module Context : sig
                 the runtime system. *)
       ; dynamically_linked_foreign_archives : bool
       ; instrument_with : Lib_name.t list
+      ; merlin : bool
       }
   end
 
@@ -40,7 +41,6 @@ module Context : sig
                 is left opaque as we leave to opam to interpret it. *)
       ; switch : string
       ; root : string option
-      ; merlin : bool
       }
   end
 
@@ -80,7 +80,7 @@ val init :
      ?x:Context_name.t
   -> ?profile:Profile.t
   -> ?instrument_with:Lib_name.t list
-  -> ?path:Path.t
+  -> ?workspace_file:Path.t
   -> unit
   -> unit
 
