@@ -61,6 +61,9 @@ module type S = sig
 
   val to_list : t -> elt list
 
+  (** Convert to a list and map every element. *)
+  val to_list_map : t -> f:(elt -> 'a) -> 'a list
+
   val find : t -> f:(elt -> bool) -> elt option
 
   val union_all : t list -> t
@@ -71,5 +74,5 @@ module type S = sig
 
   val of_keys : _ map -> t
 
-  val to_map : t -> unit map
+  val to_map : t -> f:(elt -> 'a) -> 'a map
 end
